@@ -1,4 +1,4 @@
-/* $Id: ai_gui.hpp 23704 2012-01-01 17:22:32Z alberth $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,7 +14,7 @@
 
 #include "../company_type.h"
 
-void ShowAIDebugWindow(CompanyID show_company = INVALID_COMPANY);
+Window* ShowAIDebugWindow(CompanyID show_company = INVALID_COMPANY);
 void ShowAIConfigWindow();
 void ShowAIDebugWindowIfAIError();
 void InitializeAIGui();
