@@ -1,4 +1,4 @@
-/* $Id: cargotype.cpp 26482 2014-04-23 20:13:33Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -28,12 +28,12 @@ CargoSpec CargoSpec::array[NUM_CARGO];
  * Bitmask of cargo types available. This includes phony cargoes like regearing cargoes.
  * Initialized during a call to #SetupCargoForClimate.
  */
-uint32 _cargo_mask;
+CargoTypes _cargo_mask;
 
 /**
  * Bitmask of real cargo types available. Phony cargoes like regearing cargoes are excluded.
  */
-uint32 _standard_cargo_mask;
+CargoTypes _standard_cargo_mask;
 
 /**
  * Set up the default cargo types for the given landscape type.
