@@ -1,4 +1,4 @@
-/* $Id: company_cmd.cpp 23822 2012-01-17 21:13:02Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -34,9 +34,15 @@
 #include "vehicle_func.h"
 #include "smallmap_gui.h"
 #include "game/game.hpp"
+#include "goal_base.h"
+#include "story_base.h"
 
 #include "table/strings.h"
 
+#include "safeguards.h"
+
+void ClearEnginesHiddenFlagOfCompany(CompanyID cid);
+
 CompanyByte _local_company;   ///< Company controlled by the human player at this client. Can also be #COMPANY_SPECTATOR.
 CompanyByte _current_company; ///< Company currently doing an action.
 Colours _company_colours[MAX_COMPANIES];  ///< NOSAVE: can be determined from company structs.
@@ -59,6 +65,7 @@ Company::Company(uint16 name_1, bool is_ai)
 	this->is_ai = is_ai;
 	this->terraform_limit = _settings_game.construction.terraform_frame_burst << 16;
 	this->clear_limit     = _settings_game.construction.clear_frame_burst << 16;
+	this->tree_limit      = _settings_game.construction.tree_frame_burst << 16;
 
 	for (uint j = 0; j < 4; j++) this->share_owners[j] = COMPANY_SPECTATOR;
 	InvalidateWindowData(WC_PERFORMANCE_DETAIL, 0, INVALID_COMPANY);
@@ -81,6 +88,7 @@ void Company::PostDestructor(size_t index)
 	InvalidateWindowData(WC_GRAPH_LEGEND, 0, (int)index);
 	InvalidateWindowData(WC_PERFORMANCE_DETAIL, 0, (int)index);
 	InvalidateWindowData(WC_COMPANY_LEAGUE, 0, 0);
+	InvalidateWindowData(WC_LINKGRAPH_LEGEND, 0);
 	/* If the currently shown error message has this company in it, then close it. */
 	InvalidateWindowData(WC_ERRMSG, 0);
 }
@@ -96,9 +104,12 @@ void SetLocalCompany(CompanyID new_company)
 	/* company could also be COMPANY_SPECTATOR or OWNER_NONE */
 	assert(Company::IsValidID(new_company) || new_company == COMPANY_SPECTATOR || new_company == OWNER_NONE);
 
+	/* If actually changing to another company, several windows need closing */
+	bool switching_company = _local_company != new_company;
+
 #ifdef ENABLE_NETWORK
 	/* Delete the chat window, if you were team chatting. */
-	InvalidateWindowData(WC_SEND_NETWORK_MSG, DESTTYPE_TEAM, _local_company);
+	if (switching_company) InvalidateWindowData(WC_SEND_NETWORK_MSG, DESTTYPE_TEAM, _local_company);
 #endif
 
 	assert(IsLocalCompany());
@@ -106,7 +117,7 @@ void SetLocalCompany(CompanyID new_company)
 	_current_company = _local_company = new_company;
 
 	/* Delete any construction windows... */
-	DeleteConstructionWindows();
+	if (switching_company) DeleteConstructionWindows();
 
 	/* ... and redraw the whole screen. */
 	MarkWholeScreenDirty();
@@ -154,7 +165,7 @@ static bool IsValidCompanyManagerFace(CompanyManagerFace cmf)
 	for (CompanyManagerFaceVariable cmfv = CMFV_CHEEKS; cmfv < CMFV_END; cmfv++) {
 		switch (cmfv) {
 			case CMFV_MOUSTACHE:   if (!has_moustache)   continue; break;
-			case CMFV_LIPS:        // FALL THROUGH
+			case CMFV_LIPS:
 			case CMFV_NOSE:        if (has_moustache)    continue; break;
 			case CMFV_TIE_EARRING: if (!has_tie_earring) continue; break;
 			case CMFV_GLASSES:     if (!has_glasses)     continue; break;
@@ -180,7 +191,7 @@ void InvalidateCompanyWindows(const Company *company)
 
 /**
  * Verify whether the company can pay the bill.
- * @param cost [inout] Money to pay, is changed to an error if the company does not have enough money.
+ * @param[in,out] cost Money to pay, is changed to an error if the company does not have enough money.
  * @return Function returns \c true if the company has enough money, else it returns \c false.
  */
 bool CheckCompanyHasMoney(CommandCost &cost)
@@ -260,6 +271,7 @@ void UpdateLandscapingLimits()
 	FOR_ALL_COMPANIES(c) {
 		c->terraform_limit = min(c->terraform_limit + _settings_game.construction.terraform_per_64k_frames, (uint32)_settings_game.construction.terraform_frame_burst << 16);
 		c->clear_limit     = min(c->clear_limit     + _settings_game.construction.clear_per_64k_frames,     (uint32)_settings_game.construction.clear_frame_burst << 16);
+		c->tree_limit      = min(c->tree_limit      + _settings_game.construction.tree_per_64k_frames,      (uint32)_settings_game.construction.tree_frame_burst << 16);
 	}
 }
 
@@ -373,7 +385,7 @@ set_name:;
 			SetDParam(1, STR_NEWS_COMPANY_LAUNCH_DESCRIPTION);
 			SetDParamStr(2, cni->company_name);
 			SetDParam(3, t->index);
-			AddNewsItem(STR_MESSAGE_NEWS_FORMAT, NS_COMPANY_NEW, NR_TILE, c->last_build_coordinate, NR_NONE, UINT32_MAX, cni);
+			AddNewsItem(STR_MESSAGE_NEWS_FORMAT, NT_COMPANY_INFO, NF_COMPANY, NR_TILE, c->last_build_coordinate, NR_NONE, UINT32_MAX, cni);
 		}
 		return;
 	}
@@ -508,10 +520,19 @@ restart:;
 void ResetCompanyLivery(Company *c)
 {
 	for (LiveryScheme scheme = LS_BEGIN; scheme < LS_END; scheme++) {
-		c->livery[scheme].in_use  = false;
+		c->livery[scheme].in_use  = 0;
 		c->livery[scheme].colour1 = c->colour;
 		c->livery[scheme].colour2 = c->colour;
 	}
+
+	Group *g;
+	FOR_ALL_GROUPS(g) {
+		if (g->owner == c->index) {
+			g->livery.in_use  = 0;
+			g->livery.colour1 = c->colour;
+			g->livery.colour2 = c->colour;
+		}
+	}
 }
 
 /**
@@ -548,15 +569,17 @@ Company *DoStartupNewCompany(bool is_ai, CompanyID company = INVALID_COMPANY)
 	c->avail_railtypes = GetCompanyRailtypes(c->index);
 	c->avail_roadtypes = GetCompanyRoadtypes(c->index);
 	c->inaugurated_year = _cur_year;
-	RandomCompanyManagerFaceBits(c->face, (GenderEthnicity)Random(), false); // create a random company manager face
+	RandomCompanyManagerFaceBits(c->face, (GenderEthnicity)Random(), false, false); // create a random company manager face
 
 	SetDefaultCompanySettings(c->index);
+	ClearEnginesHiddenFlagOfCompany(c->index);
 
 	GeneratePresidentName(c);
 
 	SetWindowDirty(WC_GRAPH_LEGEND, 0);
 	SetWindowClassesDirty(WC_CLIENT_LIST_POPUP);
 	SetWindowDirty(WC_CLIENT_LIST, 0);
+	InvalidateWindowData(WC_LINKGRAPH_LEGEND, 0);
 	BuildOwnerLegend();
 	InvalidateWindowData(WC_SMALLMAP, 0, 1);
 
@@ -592,7 +615,7 @@ static void MaybeStartNewCompany()
 	if (n < (uint)_settings_game.difficulty.max_no_competitors) {
 		/* Send a command to all clients to start up a new AI.
 		 * Works fine for Multiplayer and Singleplayer */
-		DoCommandP(0, 1 | INVALID_COMPANY << 16, 0, CMD_COMPANY_CTRL);
+		DoCommandP(0, CCA_NEW_AI | INVALID_COMPANY << 16, 0, CMD_COMPANY_CTRL);
 	}
 }
 
@@ -723,9 +746,9 @@ void CompaniesYearlyLoop()
 		ShowCompanyFinances(_local_company);
 		c = Company::Get(_local_company);
 		if (c->num_valid_stat_ent > 5 && c->old_economy[0].performance_history < c->old_economy[4].performance_history) {
-			SndPlayFx(SND_01_BAD_YEAR);
+			if (_settings_client.sound.new_year) SndPlayFx(SND_01_BAD_YEAR);
 		} else {
-			SndPlayFx(SND_00_GOOD_YEAR);
+			if (_settings_client.sound.new_year) SndPlayFx(SND_00_GOOD_YEAR);
 		}
 	}
 }
@@ -784,11 +807,9 @@ void CompanyAdminRemove(CompanyID company_id, CompanyRemoveReason reason)
  * @param tile unused
  * @param flags operation to perform
  * @param p1 various functionality
- * - bits 0..15:
- *        = 0 - create a new company
- *        = 1 - create a new AI company
- *        = 2 - delete a company
- * - bits 16..24: CompanyID
+ * - bits 0..15: CompanyCtrlAction
+ * - bits 16..23: CompanyID
+ * - bits 24..31: CompanyRemoveReason (with CCA_DELETE)
  * @param p2 ClientID
  * @param text unused
  * @return the cost of this operation or an error
@@ -797,18 +818,17 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 {
 	InvalidateWindowData(WC_COMPANY_LEAGUE, 0, 0);
 	CompanyID company_id = (CompanyID)GB(p1, 16, 8);
-#ifdef ENABLE_NETWORK
-	ClientID client_id = (ClientID)p2;
-#endif /* ENABLE_NETWORK */
 
-	switch (GB(p1, 0, 16)) {
-		case 0: { // Create a new company
+	switch ((CompanyCtrlAction)GB(p1, 0, 16)) {
+		case CCA_NEW: { // Create a new company
 			/* This command is only executed in a multiplayer game */
 			if (!_networking) return CMD_ERROR;
 
 #ifdef ENABLE_NETWORK
 			/* Has the network client a correct ClientIndex? */
 			if (!(flags & DC_EXEC)) return CommandCost();
+
+			ClientID client_id = (ClientID)p2;
 			NetworkClientInfo *ci = NetworkClientInfo::GetByClientID(client_id);
 #ifndef DEBUG_DUMP_COMMANDS
 			/* When replaying the client ID is not a valid client; there
@@ -847,62 +867,24 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 				MarkWholeScreenDirty();
 			}
 
-			if (_network_server) {
-				if (ci != NULL) {
-					/* ci is only NULL when replaying.
-					 * When replaying no client is actually in need of an update. */
-					ci->client_playas = c->index;
-					NetworkUpdateClientInfo(ci->client_id);
-				}
-
-				if (Company::IsValidID(c->index)) {
-					_network_company_states[c->index].months_empty = 0;
-					_network_company_states[c->index].password[0] = '\0';
-					NetworkServerUpdateCompanyPassworded(c->index, false);
-
-					/* XXX - When a client joins, we automatically set its name to the
-					 * client's name (for some reason). As it stands now only the server
-					 * knows the client's name, so it needs to send out a "broadcast" to
-					 * do this. To achieve this we send a network command. However, it
-					 * uses _local_company to execute the command as.  To prevent abuse
-					 * (eg. only yourself can change your name/company), we 'cheat' by
-					 * impersonation _local_company as the server. Not the best solution;
-					 * but it works.
-					 * TODO: Perhaps this could be improved by when the client is ready
-					 * with joining to let it send itself the command, and not the server?
-					 * For example in network_client.c:534? */
-					if (ci != NULL) {
-						/* ci is only NULL when replaying.
-						 * When replaying, the command to rename the president will
-						 * automatically be ran, so this is not even needed to get
-						 * the exact same state. */
-						NetworkSendCommand(0, 0, 0, CMD_RENAME_PRESIDENT, NULL, ci->client_name, c->index);
-					}
-				}
-
-				/* Announce new company on network. */
-				NetworkAdminCompanyInfo(c, true);
-
-				if (ci != NULL) {
-					/* ci is only NULL when replaying.
-					 * When replaying, the message that someone started a new company
-					 * is not interesting at all. */
-					NetworkServerSendChat(NETWORK_ACTION_COMPANY_NEW, DESTTYPE_BROADCAST, 0, "", ci->client_id, c->index + 1);
-				}
-			}
+			NetworkServerNewCompany(c, ci);
 #endif /* ENABLE_NETWORK */
 			break;
 		}
 
-		case 1: // Make a new AI company
+		case CCA_NEW_AI: { // Make a new AI company
 			if (!(flags & DC_EXEC)) return CommandCost();
 
 			if (company_id != INVALID_COMPANY && (company_id >= MAX_COMPANIES || Company::IsValidID(company_id))) return CMD_ERROR;
-			DoStartupNewCompany(true, company_id);
+			Company *c = DoStartupNewCompany(true, company_id);
+#ifdef ENABLE_NETWORK
+			if (c != NULL) NetworkServerNewCompany(c, NULL);
+#endif /* ENABLE_NETWORK */
 			break;
+		}
 
-		case 2: { // Delete a company
-			CompanyRemoveReason reason = (CompanyRemoveReason)GB(p2, 0, 2);
+		case CCA_DELETE: { // Delete a company
+			CompanyRemoveReason reason = (CompanyRemoveReason)GB(p1, 24, 8);
 			if (reason >= CRR_END) return CMD_ERROR;
 
 			Company *c = Company::GetIfValid(company_id);
@@ -919,7 +901,7 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 			SetDParam(0, STR_NEWS_COMPANY_BANKRUPT_TITLE);
 			SetDParam(1, STR_NEWS_COMPANY_BANKRUPT_DESCRIPTION);
 			SetDParamStr(2, cni->company_name);
-			AddCompanyNewsItem(STR_MESSAGE_NEWS_FORMAT, NS_COMPANY_BANKRUPT, cni);
+			AddCompanyNewsItem(STR_MESSAGE_NEWS_FORMAT, cni);
 
 			/* Remove the company */
 			ChangeOwnershipOfCompanyItems(c->index, INVALID_OWNER);
@@ -930,6 +912,8 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 			AI::BroadcastNewEvent(new ScriptEventCompanyBankrupt(c_index));
 			Game::NewEvent(new ScriptEventCompanyBankrupt(c_index));
 			CompanyAdminRemove(c_index, (CompanyRemoveReason)reason);
+
+			if (StoryPage::GetNumItems() == 0 || Goal::GetNumItems() == 0) InvalidateWindowData(WC_MAIN_TOOLBAR, 0);
 			break;
 		}
 
@@ -971,23 +955,26 @@ CommandCost CmdSetCompanyManagerFace(TileIndex tile, DoCommandFlag flags, uint32
  * @param flags operation to perform
  * @param p1 bitstuffed:
  * p1 bits 0-7 scheme to set
- * p1 bits 8-9 set in use state or first/second colour
+ * p1 bit 8 set first/second colour
  * @param p2 new colour for vehicles, property, etc.
  * @param text unused
  * @return the cost of this operation or an error
  */
 CommandCost CmdSetCompanyColour(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	Colours colour = Extract<Colours, 0, 4>(p2);
+	Colours colour = Extract<Colours, 0, 8>(p2);
 	LiveryScheme scheme = Extract<LiveryScheme, 0, 8>(p1);
-	byte state = GB(p1, 8, 2);
+	bool second = HasBit(p1, 8);
 
-	if (scheme >= LS_END || state >= 3 || colour == INVALID_COLOUR) return CMD_ERROR;
+	if (scheme >= LS_END || (colour >= COLOUR_END && colour != INVALID_COLOUR)) return CMD_ERROR;
+
+	/* Default scheme can't be reset to invalid. */
+	if (scheme == LS_DEFAULT && colour == INVALID_COLOUR) return CMD_ERROR;
 
 	Company *c = Company::Get(_current_company);
 
 	/* Ensure no two companies have the same primary colour */
-	if (scheme == LS_DEFAULT && state == 0) {
+	if (scheme == LS_DEFAULT && !second) {
 		const Company *cc;
 		FOR_ALL_COMPANIES(cc) {
 			if (cc != c && cc->colour == colour) return CMD_ERROR;
@@ -995,52 +982,48 @@ CommandCost CmdSetCompanyColour(TileIndex tile, DoCommandFlag flags, uint32 p1,
 	}
 
 	if (flags & DC_EXEC) {
-		switch (state) {
-			case 0:
-				c->livery[scheme].colour1 = colour;
+		if (!second) {
+			if (scheme != LS_DEFAULT) SB(c->livery[scheme].in_use, 0, 1, colour != INVALID_COLOUR);
+			if (colour == INVALID_COLOUR) colour = (Colours)c->livery[LS_DEFAULT].colour1;
+			c->livery[scheme].colour1 = colour;
 
-				/* If setting the first colour of the default scheme, adjust the
-				 * original and cached company colours too. */
-				if (scheme == LS_DEFAULT) {
-					_company_colours[_current_company] = colour;
-					c->colour = colour;
-					CompanyAdminUpdate(c);
+			/* If setting the first colour of the default scheme, adjust the
+			 * original and cached company colours too. */
+			if (scheme == LS_DEFAULT) {
+				for (int i = 1; i < LS_END; i++) {
+					if (!HasBit(c->livery[i].in_use, 0)) c->livery[i].colour1 = colour;
 				}
-				break;
+				_company_colours[_current_company] = colour;
+				c->colour = colour;
+				CompanyAdminUpdate(c);
+			}
+		} else {
+			if (scheme != LS_DEFAULT) SB(c->livery[scheme].in_use, 1, 1, colour != INVALID_COLOUR);
+			if (colour == INVALID_COLOUR) colour = (Colours)c->livery[LS_DEFAULT].colour2;
+			c->livery[scheme].colour2 = colour;
 
-			case 1:
-				c->livery[scheme].colour2 = colour;
-				break;
+			if (scheme == LS_DEFAULT) {
+				for (int i = 1; i < LS_END; i++) {
+					if (!HasBit(c->livery[i].in_use, 1)) c->livery[i].colour2 = colour;
+				}
+			}
+		}
 
-			case 2:
-				c->livery[scheme].in_use = colour != 0;
-
-				/* Now handle setting the default scheme's in_use flag.
-				 * This is different to the other schemes, as it signifies if any
-				 * scheme is active at all. If this flag is not set, then no
-				 * processing of vehicle types occurs at all, and only the default
-				 * colours will be used. */
-
-				/* If enabling a scheme, set the default scheme to be in use too */
-				if (colour != 0) {
-					c->livery[LS_DEFAULT].in_use = true;
+		if (c->livery[scheme].in_use != 0) {
+			/* If enabling a scheme, set the default scheme to be in use too */
+			c->livery[LS_DEFAULT].in_use = 1;
+		} else {
+			/* Else loop through all schemes to see if any are left enabled.
+			 * If not, disable the default scheme too. */
+			c->livery[LS_DEFAULT].in_use = 0;
+			for (scheme = LS_DEFAULT; scheme < LS_END; scheme++) {
+				if (c->livery[scheme].in_use != 0) {
+					c->livery[LS_DEFAULT].in_use = 1;
 					break;
 				}
-
-				/* Else loop through all schemes to see if any are left enabled.
-				 * If not, disable the default scheme too. */
-				c->livery[LS_DEFAULT].in_use = false;
-				for (scheme = LS_DEFAULT; scheme < LS_END; scheme++) {
-					if (c->livery[scheme].in_use) {
-						c->livery[LS_DEFAULT].in_use = true;
-						break;
-					}
-				}
-				break;
-
-			default:
-				break;
+			}
 		}
+
 		ResetVehicleColourMap();
 		MarkWholeScreenDirty();
 
@@ -1050,6 +1033,7 @@ CommandCost CmdSetCompanyColour(TileIndex tile, DoCommandFlag flags, uint32 p1,
 		InvalidateWindowData(WC_DELIVERED_CARGO, 0);
 		InvalidateWindowData(WC_PERFORMANCE_HISTORY, 0);
 		InvalidateWindowData(WC_COMPANY_VALUE, 0);
+		InvalidateWindowData(WC_LINKGRAPH_LEGEND, 0);
 		/* The smallmap owner view also stores the company colours. */
 		BuildOwnerLegend();
 		InvalidateWindowData(WC_SMALLMAP, 0, 1);
@@ -1103,7 +1087,7 @@ CommandCost CmdRenameCompany(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 	if (flags & DC_EXEC) {
 		Company *c = Company::Get(_current_company);
 		free(c->name);
-		c->name = reset ? NULL : strdup(text);
+		c->name = reset ? NULL : stredup(text);
 		MarkWholeScreenDirty();
 		CompanyAdminUpdate(c);
 	}
@@ -1152,12 +1136,12 @@ CommandCost CmdRenamePresident(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 		if (reset) {
 			c->president_name = NULL;
 		} else {
-			c->president_name = strdup(text);
+			c->president_name = stredup(text);
 
 			if (c->name_1 == STR_SV_UNNAMED && c->name == NULL) {
 				char buf[80];
 
-				snprintf(buf, lengthof(buf), "%s Transport", text);
+				seprintf(buf, lastof(buf), "%s Transport", text);
 				DoCommand(0, 0, 0, DC_EXEC, CMD_RENAME_COMPANY, buf);
 			}
 		}
@@ -1168,3 +1152,21 @@ CommandCost CmdRenamePresident(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 
 	return CommandCost();
 }
+
+/**
+ * Get the service interval for the given company and vehicle type.
+ * @param c The company, or NULL for client-default settings.
+ * @param type The vehicle type to get the interval for.
+ * @return The service interval.
+ */
+int CompanyServiceInterval(const Company *c, VehicleType type)
+{
+	const VehicleDefaultSettings *vds = (c == NULL) ? &_settings_client.company.vehicle : &c->settings.vehicle;
+	switch (type) {
+		default: NOT_REACHED();
+		case VEH_TRAIN:    return vds->servint_trains;
+		case VEH_ROAD:     return vds->servint_roadveh;
+		case VEH_AIRCRAFT: return vds->servint_aircraft;
+		case VEH_SHIP:     return vds->servint_ships;
+	}
+}
