@@ -1,4 +1,4 @@
-/* $Id: smallvec_type.hpp 24452 2012-08-01 19:07:04Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -144,6 +144,37 @@ public:
 		return &this->data[begin];
 	}
 
+	/**
+	 * Set the size of the vector, effectively truncating items from the end or appending uninitialised ones.
+	 * @param num_items Target size.
+	 */
+	inline void Resize(uint num_items)
+	{
+		this->items = num_items;
+
+		if (this->items > this->capacity) {
+			this->capacity = Align(this->items, S);
+			this->data = ReallocT(this->data, this->capacity);
+		}
+	}
+
+	/**
+	 * Insert a new item at a specific position into the vector, moving all following items.
+	 * @param item Position at which the new item should be inserted
+	 * @return pointer to the new item
+	 */
+	inline T *Insert(T *item)
+	{
+		assert(item >= this->Begin() && item <= this->End());
+
+		size_t to_move = this->End() - item;
+		size_t start = item - this->Begin();
+
+		this->Append();
+		if (to_move > 0) MemMoveT(this->Begin() + start + 1, this->Begin() + start, to_move);
+		return this->Begin() + start;
+	}
+
 	/**
 	 * Search for the first occurrence of an item.
 	 * The '!=' operator of T is used for comparison.
@@ -178,10 +209,10 @@ public:
 	 * @param item Item to search for
 	 * @return The position of the item, or -1 when not present
 	 */
-	inline int FindIndex(const T &item)
+	inline int FindIndex(const T &item) const
 	{
 		int index = 0;
-		T *pos = this->Begin();
+		const T *pos = this->Begin();
 		const T *end = this->End();
 		while (pos != end && *pos != item) {
 			pos++;
@@ -212,6 +243,32 @@ public:
 		*item = this->data[--this->items];
 	}
 
+	/**
+	 * Remove items from the vector while preserving the order of other items.
+	 * @param pos First item to remove.
+	 * @param count Number of consecutive items to remove.
+	 */
+	void ErasePreservingOrder(uint pos, uint count = 1)
+	{
+		ErasePreservingOrder(this->data + pos, count);
+	}
+
+	/**
+	 * Remove items from the vector while preserving the order of other items.
+	 * @param item First item to remove.
+	 * @param count Number of consecutive items to remove.
+	 */
+	inline void ErasePreservingOrder(T *item, uint count = 1)
+	{
+		if (count == 0) return;
+		assert(item >= this->Begin());
+		assert(item + count <= this->End());
+
+		this->items -= count;
+		ptrdiff_t to_move = this->End() - item;
+		if (to_move > 0) MemMoveT(item, item + count, to_move);
+	}
+
 	/**
 	 * Tests whether a item is present in the vector, and appends it to the end if not.
 	 * The '!=' operator of T is used for comparison.
@@ -227,6 +284,8 @@ public:
 
 	/**
 	 * Get the number of items in the list.
+	 *
+	 * @return The number of items in the list.
 	 */
 	inline uint Length() const
 	{
