@@ -1,4 +1,4 @@
-/* $Id: debug.h 23854 2012-01-26 17:24:56Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,6 +12,8 @@
 #ifndef DEBUG_H
 #define DEBUG_H
 
+#include "cpu.h"
+
 /* Debugging messages policy:
  * These should be the severities used for direct DEBUG() calls
  * maximum debugging level should be 10 if really deep, deep
@@ -26,50 +28,41 @@
  * 6.. - extremely detailed spamming
  */
 
-#ifdef NO_DEBUG_MESSAGES
-	#define DEBUG(name, level, ...) { }
-#else /* NO_DEBUG_MESSAGES */
-	/**
-	 * Output a line of debugging information.
-	 * @param name Category
-	 * @param level Debugging level, higher levels means more detailed information.
-	 */
-	#define DEBUG(name, level, ...) if ((level) == 0 || _debug_ ## name ## _level >= (level)) debug(#name, __VA_ARGS__)
+/**
+ * Output a line of debugging information.
+ * @param name Category
+ * @param level Debugging level, higher levels means more detailed information.
+ */
+#define DEBUG(name, level, ...) if ((level) == 0 || _debug_ ## name ## _level >= (level)) debug(#name, __VA_ARGS__)
 
-	extern int _debug_driver_level;
-	extern int _debug_grf_level;
-	extern int _debug_map_level;
-	extern int _debug_misc_level;
-	extern int _debug_net_level;
-	extern int _debug_sprite_level;
-	extern int _debug_oldloader_level;
-	extern int _debug_npf_level;
-	extern int _debug_yapf_level;
-	extern int _debug_freetype_level;
-	extern int _debug_script_level;
-	extern int _debug_sl_level;
-	extern int _debug_gamelog_level;
-	extern int _debug_desync_level;
-	extern int _debug_console_level;
+extern int _debug_driver_level;
+extern int _debug_grf_level;
+extern int _debug_map_level;
+extern int _debug_misc_level;
+extern int _debug_net_level;
+extern int _debug_sprite_level;
+extern int _debug_oldloader_level;
+extern int _debug_npf_level;
+extern int _debug_yapf_level;
+extern int _debug_freetype_level;
+extern int _debug_script_level;
+extern int _debug_sl_level;
+extern int _debug_gamelog_level;
+extern int _debug_desync_level;
+extern int _debug_console_level;
 #ifdef RANDOM_DEBUG
-	extern int _debug_random_level;
+extern int _debug_random_level;
 #endif
 
-	void CDECL debug(const char *dbg, const char *format, ...) WARN_FORMAT(2, 3);
-#endif /* NO_DEBUG_MESSAGES */
+void CDECL debug(const char *dbg, const char *format, ...) WARN_FORMAT(2, 3);
 
+char *DumpDebugFacilityNames(char *buf, char *last);
 void SetDebugString(const char *s);
 const char *GetDebugString();
 
 /* Shorter form for passing filename and linenumber */
 #define FILE_LINE __FILE__, __LINE__
 
-/**
- * Get the tick counter from the CPU (high precision timing).
- * @return The count.
- */
-uint64 ottd_rdtsc();
-
 /* Used for profiling
  *
  * Usage:
@@ -82,7 +75,7 @@ uint64 ottd_rdtsc();
  *
  * for (int i = 0; i < 5; i++) {
  *   TIC();
- *     --Do yuor code--
+ *     --Do your code--
  *   TOC("A name", 5);
  * }
  *
