@@ -1,4 +1,4 @@
-/* $Id: fios.cpp 27653 2016-09-04 16:06:50Z alberth $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -20,9 +20,9 @@
 #include "string_func.h"
 #include <sys/stat.h>
 
-#ifndef WIN32
+#ifndef _WIN32
 # include <unistd.h>
-#endif /* WIN32 */
+#endif /* _WIN32 */
 
 #include "table/strings.h"
 
@@ -56,7 +56,7 @@ int CDECL CompareFiosItems(const FiosItem *da, const FiosItem *db)
 	if ((_savegame_sort_order & SORT_BY_NAME) == 0 && da->mtime != db->mtime) {
 		r = da->mtime < db->mtime ? -1 : 1;
 	} else {
-		r = strcasecmp(da->title, db->title);
+		r = strnatcmp(da->title, db->title);
 	}
 
 	if (_savegame_sort_order & SORT_DESCENDING) r = -r;
@@ -77,7 +77,7 @@ void FileList::BuildFileList(AbstractFileType abstract_filetype, SaveLoadOperati
 {
 	this->Clear();
 
-	assert(fop == SLO_LOAD || SLO_SAVE);
+	assert(fop == SLO_LOAD || fop == SLO_SAVE);
 	switch (abstract_filetype) {
 		case FT_NONE:
 			break;
@@ -153,12 +153,11 @@ const char *FiosBrowseTo(const FiosItem *item)
 {
 	switch (item->type) {
 		case FIOS_TYPE_DRIVE:
-#if defined(WINCE)
-			seprintf(_fios_path, _fios_path_last, PATHSEP "");
-#elif defined(WIN32) || defined(__OS2__)
+#if defined(_WIN32) || defined(__OS2__)
 			seprintf(_fios_path, _fios_path_last, "%c:" PATHSEP, item->title[0]);
 #endif
-			/* FALL THROUGH */
+			break;
+
 		case FIOS_TYPE_INVALID:
 			break;
 
@@ -319,14 +318,30 @@ bool FiosFileScanner::AddFile(const char *filename, size_t basepath_length, cons
 	}
 
 	FiosItem *fios = file_list.Append();
-#ifdef WIN32
-	struct _stat sb;
-	if (_tstat(OTTD2FS(filename), &sb) == 0) {
+#ifdef _WIN32
+	// Retrieve the file modified date using GetFileTime rather than stat to work around an obscure MSVC bug that affects Windows XP
+	HANDLE fh = CreateFile(OTTD2FS(filename), GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, nullptr, OPEN_EXISTING, 0, nullptr);
+
+	if (fh != INVALID_HANDLE_VALUE) {
+		FILETIME ft;
+		ULARGE_INTEGER ft_int64;
+
+		if (GetFileTime(fh, nullptr, nullptr, &ft) != 0) {
+			ft_int64.HighPart = ft.dwHighDateTime;
+			ft_int64.LowPart = ft.dwLowDateTime;
+
+			// Convert from hectonanoseconds since 01/01/1601 to seconds since 01/01/1970
+			fios->mtime = ft_int64.QuadPart / 10000000ULL - 11644473600ULL;
+		} else {
+			fios->mtime = 0;
+		}
+
+		CloseHandle(fh);
 #else
 	struct stat sb;
 	if (stat(filename, &sb) == 0) {
-#endif
 		fios->mtime = sb.st_mtime;
+#endif
 	} else {
 		fios->mtime = 0;
 	}
