@@ -1,4 +1,4 @@
-/* $Id: gfx_layout.cpp 27367 2015-08-09 12:33:27Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -21,6 +21,14 @@
 #include <unicode/ustring.h>
 #endif /* WITH_ICU_LAYOUT */
 
+#ifdef WITH_UNISCRIBE
+#include "os/windows/string_uniscribe.h"
+#endif /* WITH_UNISCRIBE */
+
+#ifdef WITH_COCOA
+#include "os/macosx/string_osx.h"
+#endif
+
 #include "safeguards.h"
 
 
@@ -113,32 +121,18 @@ le_bool Font::getGlyphPoint(LEGlyphID glyph, le_int32 pointNumber, LEPoint &poin
 	return FALSE;
 }
 
-static size_t AppendToBuffer(UChar *buff, const UChar *buffer_last, WChar c)
-{
-	/* Transform from UTF-32 to internal ICU format of UTF-16. */
-	int32 length = 0;
-	UErrorCode err = U_ZERO_ERROR;
-	u_strFromUTF32(buff, buffer_last - buff, &length, (UChar32*)&c, 1, &err);
-	return length;
-}
-
 /**
  * Wrapper for doing layouts with ICU.
  */
 class ICUParagraphLayout : public AutoDeleteSmallVector<ParagraphLayouter::Line *, 4>, public ParagraphLayouter {
-	ParagraphLayout *p; ///< The actual ICU paragraph layout.
+	icu::ParagraphLayout *p; ///< The actual ICU paragraph layout.
 public:
-	/** Helper for GetLayouter, to get the right type. */
-	typedef UChar CharType;
-	/** Helper for GetLayouter, to get whether the layouter supports RTL. */
-	static const bool SUPPORTS_RTL = true;
-
 	/** Visual run contains data about the bit of text with the same font. */
 	class ICUVisualRun : public ParagraphLayouter::VisualRun {
-		const ParagraphLayout::VisualRun *vr; ///< The actual ICU vr.
+		const icu::ParagraphLayout::VisualRun *vr; ///< The actual ICU vr.
 
 	public:
-		ICUVisualRun(const ParagraphLayout::VisualRun *vr) : vr(vr) { }
+		ICUVisualRun(const icu::ParagraphLayout::VisualRun *vr) : vr(vr) { }
 
 		const Font *GetFont() const          { return (const Font*)vr->getFont(); }
 		int GetGlyphCount() const            { return vr->getGlyphCount(); }
@@ -150,10 +144,10 @@ public:
 
 	/** A single line worth of VisualRuns. */
 	class ICULine : public AutoDeleteSmallVector<ICUVisualRun *, 4>, public ParagraphLayouter::Line {
-		ParagraphLayout::Line *l; ///< The actual ICU line.
+		icu::ParagraphLayout::Line *l; ///< The actual ICU line.
 
 	public:
-		ICULine(ParagraphLayout::Line *l) : l(l)
+		ICULine(icu::ParagraphLayout::Line *l) : l(l)
 		{
 			for (int i = 0; i < l->countRuns(); i++) {
 				*this->Append() = new ICUVisualRun(l->getVisualRun(i));
@@ -173,46 +167,65 @@ public:
 		}
 	};
 
-	ICUParagraphLayout(ParagraphLayout *p) : p(p) { }
+	ICUParagraphLayout(icu::ParagraphLayout *p) : p(p) { }
 	~ICUParagraphLayout() { delete p; }
 	void Reflow() { p->reflow(); }
 
 	ParagraphLayouter::Line *NextLine(int max_width)
 	{
-		ParagraphLayout::Line *l = p->nextLine(max_width);
+		icu::ParagraphLayout::Line *l = p->nextLine(max_width);
 		return l == NULL ? NULL : new ICULine(l);
 	}
 };
 
-static ParagraphLayouter *GetParagraphLayout(UChar *buff, UChar *buff_end, FontMap &fontMapping)
-{
-	int32 length = buff_end - buff;
+/**
+ * Helper class to construct a new #ICUParagraphLayout.
+ */
+class ICUParagraphLayoutFactory {
+public:
+	/** Helper for GetLayouter, to get the right type. */
+	typedef UChar CharType;
+	/** Helper for GetLayouter, to get whether the layouter supports RTL. */
+	static const bool SUPPORTS_RTL = true;
 
-	if (length == 0) {
-		/* ICU's ParagraphLayout cannot handle empty strings, so fake one. */
-		buff[0] = ' ';
-		length = 1;
-		fontMapping.End()[-1].first++;
+	static ParagraphLayouter *GetParagraphLayout(UChar *buff, UChar *buff_end, FontMap &fontMapping)
+	{
+		int32 length = buff_end - buff;
+
+		if (length == 0) {
+			/* ICU's ParagraphLayout cannot handle empty strings, so fake one. */
+			buff[0] = ' ';
+			length = 1;
+			fontMapping.End()[-1].first++;
+		}
+
+		/* Fill ICU's FontRuns with the right data. */
+		icu::FontRuns runs(fontMapping.Length());
+		for (FontMap::iterator iter = fontMapping.Begin(); iter != fontMapping.End(); iter++) {
+			runs.add(iter->second, iter->first);
+		}
+
+		LEErrorCode status = LE_NO_ERROR;
+		/* ParagraphLayout does not copy "buff", so it must stay valid.
+		 * "runs" is copied according to the ICU source, but the documentation does not specify anything, so this might break somewhen. */
+		icu::ParagraphLayout *p = new icu::ParagraphLayout(buff, length, &runs, NULL, NULL, NULL, _current_text_dir == TD_RTL ? 1 : 0, false, status);
+		if (status != LE_NO_ERROR) {
+			delete p;
+			return NULL;
+		}
+
+		return new ICUParagraphLayout(p);
 	}
 
-	/* Fill ICU's FontRuns with the right data. */
-	FontRuns runs(fontMapping.Length());
-	for (FontMap::iterator iter = fontMapping.Begin(); iter != fontMapping.End(); iter++) {
-		runs.add(iter->second, iter->first);
+	static size_t AppendToBuffer(UChar *buff, const UChar *buffer_last, WChar c)
+	{
+		/* Transform from UTF-32 to internal ICU format of UTF-16. */
+		int32 length = 0;
+		UErrorCode err = U_ZERO_ERROR;
+		u_strFromUTF32(buff, buffer_last - buff, &length, (UChar32*)&c, 1, &err);
+		return length;
 	}
-
-	LEErrorCode status = LE_NO_ERROR;
-	/* ParagraphLayout does not copy "buff", so it must stay valid.
-	 * "runs" is copied according to the ICU source, but the documentation does not specify anything, so this might break somewhen. */
-	ParagraphLayout *p = new ParagraphLayout(buff, length, &runs, NULL, NULL, NULL, _current_text_dir == TD_RTL ? UBIDI_DEFAULT_RTL : UBIDI_DEFAULT_LTR, false, status);
-	if (status != LE_NO_ERROR) {
-		delete p;
-		return NULL;
-	}
-
-	return new ICUParagraphLayout(p);
-}
-
+};
 #endif /* WITH_ICU_LAYOUT */
 
 /*** Paragraph layout ***/
@@ -236,11 +249,6 @@ static ParagraphLayouter *GetParagraphLayout(UChar *buff, UChar *buff_end, FontM
  */
 class FallbackParagraphLayout : public ParagraphLayouter {
 public:
-	/** Helper for GetLayouter, to get the right type. */
-	typedef WChar CharType;
-	/** Helper for GetLayouter, to get whether the layouter supports RTL. */
-	static const bool SUPPORTS_RTL = false;
-
 	/** Visual run contains data about the bit of text with the same font. */
 	class FallbackVisualRun : public ParagraphLayouter::VisualRun {
 		Font *font;       ///< The font used to layout these.
@@ -280,6 +288,42 @@ public:
 	const ParagraphLayouter::Line *NextLine(int max_width);
 };
 
+/**
+ * Helper class to construct a new #FallbackParagraphLayout.
+ */
+class FallbackParagraphLayoutFactory {
+public:
+	/** Helper for GetLayouter, to get the right type. */
+	typedef WChar CharType;
+	/** Helper for GetLayouter, to get whether the layouter supports RTL. */
+	static const bool SUPPORTS_RTL = false;
+
+	/**
+	 * Get the actual ParagraphLayout for the given buffer.
+	 * @param buff The begin of the buffer.
+	 * @param buff_end The location after the last element in the buffer.
+	 * @param fontMapping THe mapping of the fonts.
+	 * @return The ParagraphLayout instance.
+	 */
+	static ParagraphLayouter *GetParagraphLayout(WChar *buff, WChar *buff_end, FontMap &fontMapping)
+	{
+		return new FallbackParagraphLayout(buff, buff_end - buff, fontMapping);
+	}
+
+	/**
+	 * Append a wide character to the internal buffer.
+	 * @param buff        The buffer to append to.
+	 * @param buffer_last The end of the buffer.
+	 * @param c           The character to add.
+	 * @return The number of buffer spaces that were used.
+	 */
+	static size_t AppendToBuffer(WChar *buff, const WChar *buffer_last, WChar c)
+	{
+		*buff = c;
+		return 1;
+	}
+};
+
 /**
  * Create the visual run.
  * @param font       The font to use for this run.
@@ -458,11 +502,6 @@ const ParagraphLayouter::Line *FallbackParagraphLayout::NextLine(int max_width)
 		return l;
 	}
 
-	const WChar *begin = this->buffer;
-	const WChar *last_space = NULL;
-	const WChar *last_char = begin;
-	int width = 0;
-
 	int offset = this->buffer - this->buffer_begin;
 	FontMap::iterator iter = this->runs.Begin();
 	while (iter->first <= offset) {
@@ -473,6 +512,10 @@ const ParagraphLayouter::Line *FallbackParagraphLayout::NextLine(int max_width)
 	const FontCache *fc = iter->second->fc;
 	const WChar *next_run = this->buffer_begin + iter->first;
 
+	const WChar *begin = this->buffer;
+	const WChar *last_space = NULL;
+	const WChar *last_char;
+	int width = 0;
 	for (;;) {
 		WChar c = *this->buffer;
 		last_char = this->buffer;
@@ -536,31 +579,6 @@ const ParagraphLayouter::Line *FallbackParagraphLayout::NextLine(int max_width)
 	return l;
 }
 
-/**
- * Appand a wide character to the internal buffer.
- * @param buff        The buffer to append to.
- * @param buffer_last The end of the buffer.
- * @param c           The character to add.
- * @return The number of buffer spaces that were used.
- */
-static size_t AppendToBuffer(WChar *buff, const WChar *buffer_last, WChar c)
-{
-	*buff = c;
-	return 1;
-}
-
-/**
- * Get the actual ParagraphLayout for the given buffer.
- * @param buff The begin of the buffer.
- * @param buff_end The location after the last element in the buffer.
- * @param fontMapping THe mapping of the fonts.
- * @return The ParagraphLayout instance.
- */
-static FallbackParagraphLayout *GetParagraphLayout(WChar *buff, WChar *buff_end, FontMap &fontMapping)
-{
-	return new FallbackParagraphLayout(buff, buff_end - buff, fontMapping);
-}
-
 /**
  * Helper for getting a ParagraphLayouter of the given type.
  *
@@ -582,6 +600,7 @@ static inline void GetLayouter(Layouter::LineCacheItem &line, const char *&str,
 	Font *f = Layouter::GetFont(state.fontsize, state.cur_colour);
 
 	line.buffer = buff_begin;
+	fontMapping.Clear();
 
 	/*
 	 * Go through the whole string while adding Font instances to the font map
@@ -594,18 +613,18 @@ static inline void GetLayouter(Layouter::LineCacheItem &line, const char *&str,
 			break;
 		} else if (c >= SCC_BLUE && c <= SCC_BLACK) {
 			state.SetColour((TextColour)(c - SCC_BLUE));
-		} else if (c == SCC_PREVIOUS_COLOUR) { // Revert to the previous colour.
-			state.SetPreviousColour();
-		} else if (c == SCC_TINYFONT) {
-			state.SetFontSize(FS_SMALL);
-		} else if (c == SCC_BIGFONT) {
-			state.SetFontSize(FS_LARGE);
+		} else if (c == SCC_PUSH_COLOUR) {
+			state.PushColour();
+		} else if (c == SCC_POP_COLOUR) {
+			state.PopColour();
+		} else if (c >= SCC_FIRST_FONT && c <= SCC_LAST_FONT) {
+			state.SetFontSize((FontSize)(c - SCC_FIRST_FONT));
 		} else {
 			/* Filter out text direction characters that shouldn't be drawn, and
 			 * will not be handled in the fallback non ICU case because they are
 			 * mostly needed for RTL languages which need more ICU support. */
 			if (!T::SUPPORTS_RTL && IsTextDirectionChar(c)) continue;
-			buff += AppendToBuffer(buff, buffer_last, c);
+			buff += T::AppendToBuffer(buff, buffer_last, c);
 			continue;
 		}
 
@@ -621,7 +640,7 @@ static inline void GetLayouter(Layouter::LineCacheItem &line, const char *&str,
 	if (!fontMapping.Contains(buff - buff_begin)) {
 		fontMapping.Insert(buff - buff_begin, f);
 	}
-	line.layout = GetParagraphLayout(buff_begin, buff, fontMapping);
+	line.layout = T::GetParagraphLayout(buff_begin, buff, fontMapping);
 	line.state_after = state;
 }
 
@@ -654,11 +673,13 @@ Layouter::Layouter(const char *str, int maxw, TextColour colour, FontSize fontsi
 			line.layout->Reflow();
 		} else {
 			/* Line is new, layout it */
-#ifdef WITH_ICU_LAYOUT
 			FontState old_state = state;
+#if defined(WITH_ICU_LAYOUT) || defined(WITH_UNISCRIBE) || defined(WITH_COCOA)
 			const char *old_str = str;
+#endif
 
-			GetLayouter<ICUParagraphLayout>(line, str, state);
+#ifdef WITH_ICU_LAYOUT
+			GetLayouter<ICUParagraphLayoutFactory>(line, str, state);
 			if (line.layout == NULL) {
 				static bool warned = false;
 				if (!warned) {
@@ -668,11 +689,32 @@ Layouter::Layouter(const char *str, int maxw, TextColour colour, FontSize fontsi
 
 				state = old_state;
 				str = old_str;
-				GetLayouter<FallbackParagraphLayout>(line, str, state);
 			}
-#else
-			GetLayouter<FallbackParagraphLayout>(line, str, state);
 #endif
+
+#ifdef WITH_UNISCRIBE
+			if (line.layout == NULL) {
+				GetLayouter<UniscribeParagraphLayoutFactory>(line, str, state);
+				if (line.layout == NULL) {
+					state = old_state;
+					str = old_str;
+				}
+			}
+#endif
+
+#ifdef WITH_COCOA
+			if (line.layout == NULL) {
+				GetLayouter<CoreTextParagraphLayoutFactory>(line, str, state);
+				if (line.layout == NULL) {
+					state = old_state;
+					str = old_str;
+				}
+			}
+#endif
+
+			if (line.layout == NULL) {
+				GetLayouter<FallbackParagraphLayoutFactory>(line, str, state);
+			}
 		}
 
 		/* Copy all lines into a local cache so we can reuse them later on more easily. */
@@ -809,6 +851,13 @@ void Layouter::ResetFontCache(FontSize size)
 
 	/* We must reset the linecache since it references the just freed fonts */
 	ResetLineCache();
+
+#if defined(WITH_UNISCRIBE)
+	UniscribeResetScriptCache(size);
+#endif
+#if defined(WITH_COCOA)
+	MacOSResetScriptCache(size);
+#endif
 }
 
 /**
