@@ -1,4 +1,4 @@
-/* $Id: gfx_type.h 23446 2011-12-08 18:13:29Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -115,29 +115,41 @@ struct AnimCursor {
 
 /** Collection of variables for cursor-display and -animation */
 struct CursorVars {
-	Point pos, size, offs, delta; ///< position, size, offset from top-left, and movement
-	Point draw_pos, draw_size;    ///< position and size bounding-box for drawing
-	int short_vehicle_offset;     ///< offset of the X for short vehicles
-	CursorID sprite; ///< current image of cursor
-	PaletteID pal;
-
-	int wheel;       ///< mouse wheel movement
+	/* Logical mouse position */
+	Point pos;                    ///< logical mouse position
+	Point delta;                  ///< relative mouse movement in this tick
+	int wheel;                    ///< mouse wheel movement
+	bool fix_at;                  ///< mouse is moving, but cursor is not (used for scrolling)
 
 	/* We need two different vars to keep track of how far the scrollwheel moved.
 	 * OSX uses this for scrolling around the map. */
 	int v_wheel;
 	int h_wheel;
 
+	/* Mouse appearance */
+	PalSpriteID sprite_seq[16];   ///< current image of cursor
+	Point sprite_pos[16];         ///< relative position of individual sprites
+	uint sprite_count;            ///< number of sprites to draw
+	Point total_offs, total_size; ///< union of sprite properties
+
+	Point draw_pos, draw_size;    ///< position and size bounding-box for drawing
+
 	const AnimCursor *animate_list; ///< in case of animated cursor, list of frames
 	const AnimCursor *animate_cur;  ///< in case of animated cursor, current frame
 	uint animate_timeout;           ///< in case of animated cursor, number of ticks to show the current cursor
 
-	bool visible;    ///< cursor is visible
-	bool dirty;      ///< the rect occupied by the mouse is dirty (redraw)
-	bool fix_at;     ///< mouse is moving, but cursor is not (used for scrolling)
-	bool in_window;  ///< mouse inside this window, determines drawing logic
+	bool visible;                 ///< cursor is visible
+	bool dirty;                   ///< the rect occupied by the mouse is dirty (redraw)
+	bool in_window;               ///< mouse inside this window, determines drawing logic
 
-	bool vehchain;   ///< vehicle chain is dragged
+	/* Drag data */
+	bool vehchain;                ///< vehicle chain is dragged
+
+	bool UpdateCursorPosition(int x, int y, bool queued_warp);
+
+private:
+	bool queued_warp;
+	Point last_position;
 };
 
 /** Data about how and where to blit pixels. */
@@ -158,8 +170,35 @@ union Colour {
 		uint8 b, g, r, a; ///< colour channels in LE order
 #endif /* TTD_ENDIAN == TTD_BIG_ENDIAN */
 	};
+
+	/**
+	 * Create a new colour.
+	 * @param r The channel for the red colour.
+	 * @param g The channel for the green colour.
+	 * @param b The channel for the blue colour.
+	 * @param a The channel for the alpha/transparency.
+	 */
+	Colour(uint8 r, uint8 g, uint8 b, uint8 a = 0xFF) :
+#if TTD_ENDIAN == TTD_BIG_ENDIAN
+		a(a), r(r), g(g), b(b)
+#else
+		b(b), g(g), r(r), a(a)
+#endif /* TTD_ENDIAN == TTD_BIG_ENDIAN */
+	{
+	}
+
+	/**
+	 * Create a new colour.
+	 * @param data The colour in the correct packed format.
+	 */
+	Colour(uint data = 0) : data(data)
+	{
+	}
 };
 
+assert_compile(sizeof(Colour) == sizeof(uint32));
+
+
 /** Available font sizes */
 enum FontSize {
 	FS_NORMAL, ///< Index of the normal font in the font tables.
@@ -202,9 +241,9 @@ enum Colours {
 	COLOUR_END,
 	INVALID_COLOUR = 0xFF,
 };
-template <> struct EnumPropsT<Colours> : MakeEnumPropsT<Colours, byte, COLOUR_BEGIN, COLOUR_END, INVALID_COLOUR, 4> {};
+template <> struct EnumPropsT<Colours> : MakeEnumPropsT<Colours, byte, COLOUR_BEGIN, COLOUR_END, INVALID_COLOUR, 8> {};
 
-/** Colour of the strings, see _string_colourmap in table/palettes.h or docs/ottd-colourtext-palette.png */
+/** Colour of the strings, see _string_colourmap in table/string_colours.h or docs/ottd-colourtext-palette.png */
 enum TextColour {
 	TC_BEGIN       = 0x00,
 	TC_FROMSTRING  = 0x00,
@@ -273,4 +312,11 @@ struct Palette {
 	int count_dirty;     ///< The number of dirty elements.
 };
 
+/** Modes for 8bpp support */
+enum Support8bpp {
+	S8BPP_NONE = 0, ///< No support for 8bpp by OS or hardware, force 32bpp blitters.
+	S8BPP_SYSTEM,   ///< No 8bpp support by hardware, do not try to use 8bpp video modes or hardware palettes.
+	S8BPP_HARDWARE, ///< Full 8bpp support by OS and hardware.
+};
+
 #endif /* GFX_TYPE_H */
