@@ -1,4 +1,4 @@
-/* $Id: qtmidi.cpp 26482 2014-04-23 20:13:33Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -30,7 +30,9 @@
 
 #include "../stdafx.h"
 #include "qtmidi.h"
+#include "midifile.hpp"
 #include "../debug.h"
+#include "../base_media_base.h"
 
 #define Rect  OTTDRect
 #define Point OTTDPoint
@@ -242,7 +244,7 @@ void MusicDriver_QtMidi::Stop()
 
 		case QT_STATE_PLAY:
 			StopSong();
-			/* FALL THROUGH */
+			FALLTHROUGH;
 
 		case QT_STATE_STOP:
 			DisposeMovie(_quicktime_movie);
@@ -258,30 +260,33 @@ void MusicDriver_QtMidi::Stop()
  *
  * @param filename Path to a MIDI file.
  */
-void MusicDriver_QtMidi::PlaySong(const char *filename)
+void MusicDriver_QtMidi::PlaySong(const MusicSongInfo &song)
 {
 	if (!_quicktime_started) return;
 
-	DEBUG(driver, 2, "qtmidi: trying to play '%s'", filename);
+	std::string filename = MidiFile::GetSMFFile(song);
+	if (filename.empty()) return;
+
+	DEBUG(driver, 2, "qtmidi: trying to play '%s'", filename.c_str());
 	switch (_quicktime_state) {
 		case QT_STATE_PLAY:
 			StopSong();
 			DEBUG(driver, 3, "qtmidi: previous tune stopped");
-			/* FALL THROUGH */
+			FALLTHROUGH;
 
 		case QT_STATE_STOP:
 			DisposeMovie(_quicktime_movie);
 			DEBUG(driver, 3, "qtmidi: previous tune disposed");
 			_quicktime_state = QT_STATE_IDLE;
-			/* FALL THROUGH */
+			FALLTHROUGH;
 
 		case QT_STATE_IDLE:
-			LoadMovieForMIDIFile(filename, &_quicktime_movie);
+			LoadMovieForMIDIFile(filename.c_str(), &_quicktime_movie);
 			SetMovieVolume(_quicktime_movie, VOLUME);
 			StartMovie(_quicktime_movie);
 			_quicktime_state = QT_STATE_PLAY;
 	}
-	DEBUG(driver, 3, "qtmidi: playing '%s'", filename);
+	DEBUG(driver, 3, "qtmidi: playing '%s'", filename.c_str());
 }
 
 
@@ -294,7 +299,7 @@ void MusicDriver_QtMidi::StopSong()
 
 	switch (_quicktime_state) {
 		case QT_STATE_IDLE:
-			/* FALL THROUGH */
+			FALLTHROUGH;
 
 		case QT_STATE_STOP:
 			DEBUG(driver, 3, "qtmidi: stop requested, but already idle");
