@@ -1,4 +1,4 @@
-/* $Id: network_gui.cpp 27653 2016-09-04 16:06:50Z alberth $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -31,6 +31,7 @@
 #include "../core/geometry_func.hpp"
 #include "../genworld.h"
 #include "../map_type.h"
+#include "../guitimer_func.h"
 
 #include "../widgets/network_widget.h"
 
@@ -234,6 +235,7 @@ protected:
 	Scrollbar *vscroll;           ///< vertical scrollbar of the list of servers
 	QueryString name_editbox;     ///< Client name editbox.
 	QueryString filter_editbox;   ///< Editbox for filter on servers
+	GUITimer requery_timer;       ///< Timer for network requery
 
 	int lock_offset; ///< Left offset for lock icon.
 	int blot_offset; ///< Left offset for green/yellow/red compatibility icon.
@@ -480,6 +482,8 @@ public:
 		this->server = this->last_joined;
 		if (this->last_joined != NULL) NetworkUDPQueryServer(this->last_joined->address);
 
+		this->requery_timer.SetInterval(MILLISECONDS_PER_TICK);
+
 		this->servers.SetListing(this->last_sorting);
 		this->servers.SetSortFuncs(this->sorter_funcs);
 		this->servers.SetFilterFuncs(this->filter_funcs);
@@ -903,8 +907,11 @@ public:
 		this->vscroll->SetCapacityFromWidget(this, WID_NG_MATRIX);
 	}
 
-	virtual void OnTick()
+	virtual void OnRealtimeTick(uint delta_ms)
 	{
+		if (!this->requery_timer.Elapsed(delta_ms)) return;
+		this->requery_timer.SetInterval(MILLISECONDS_PER_TICK);
+
 		NetworkGameListRequery();
 	}
 };
@@ -1843,7 +1850,7 @@ static void PopupClientList(ClientID client_id, int x, int y)
 static const NWidgetPart _nested_client_list_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_GREY),
-		NWidget(WWT_CAPTION, COLOUR_GREY), SetDataTip(STR_NETWORK_COMPANY_LIST_CLIENT_LIST, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
+		NWidget(WWT_CAPTION, COLOUR_GREY), SetDataTip(STR_NETWORK_COMPANY_LIST_CLIENT_LIST_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(WWT_PANEL, COLOUR_GREY, WID_CL_PANEL), SetMinimalSize(250, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM), SetResize(1, 1), EndContainer(),
@@ -2053,7 +2060,8 @@ struct NetworkJoinStatusWindow : Window {
 					progress = 15; // We don't have the final size yet; the server is still compressing!
 					break;
 				}
-				/* FALL THROUGH */
+				FALLTHROUGH;
+
 			default: // Waiting is 15%, so the resting receivement of map is maximum 70%
 				progress = 15 + _network_join_bytes * (100 - 15) / _network_join_bytes_total;
 		}
@@ -2182,7 +2190,7 @@ struct NetworkCompanyPasswordWindow : public Window {
 		switch (widget) {
 			case WID_NCP_OK:
 				this->OnOk();
-				/* FALL THROUGH */
+				FALLTHROUGH;
 
 			case WID_NCP_CANCEL:
 				delete this;
