@@ -1,4 +1,4 @@
-/* $Id: network_server.cpp 24232 2012-05-12 07:47:00Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -33,6 +33,8 @@
 #include "../core/random_func.hpp"
 #include "../rev.h"
 
+#include "../safeguards.h"
+
 
 /* This file handles all the server-commands */
 
@@ -57,30 +59,96 @@ struct PacketWriter : SaveFilter {
 	ServerNetworkGameSocketHandler *cs; ///< Socket we are associated with.
 	Packet *current;                    ///< The packet we're currently writing to.
 	size_t total_size;                  ///< Total size of the compressed savegame.
+	Packet *packets;                    ///< Packet queue of the savegame; send these "slowly" to the client.
+	ThreadMutex *mutex;                 ///< Mutex for making threaded saving safe.
 
 	/**
 	 * Create the packet writer.
 	 * @param cs The socket handler we're making the packets for.
 	 */
-	PacketWriter(ServerNetworkGameSocketHandler *cs) : SaveFilter(NULL), cs(cs), current(NULL), total_size(0)
+	PacketWriter(ServerNetworkGameSocketHandler *cs) : SaveFilter(NULL), cs(cs), current(NULL), total_size(0), packets(NULL)
 	{
-		this->cs->savegame_mutex = ThreadMutex::New();
+		this->mutex = ThreadMutex::New();
 	}
 
 	/** Make sure everything is cleaned up. */
 	~PacketWriter()
 	{
-		/* Prevent double frees. */
-		if (this->cs != NULL) {
-			if (this->cs->savegame_mutex != NULL) this->cs->savegame_mutex->BeginCritical();
-			this->cs->savegame = NULL;
-			if (this->cs->savegame_mutex != NULL) this->cs->savegame_mutex->EndCritical();
+		if (this->mutex != NULL) this->mutex->BeginCritical();
 
-			delete this->cs->savegame_mutex;
-			this->cs->savegame_mutex = NULL;
+		if (this->cs != NULL && this->mutex != NULL) {
+			this->mutex->WaitForSignal();
+		}
+
+		/* This must all wait until the Destroy function is called. */
+
+		while (this->packets != NULL) {
+			Packet *p = this->packets->next;
+			delete this->packets;
+			this->packets = p;
 		}
 
 		delete this->current;
+
+		if (this->mutex != NULL) this->mutex->EndCritical();
+
+		delete this->mutex;
+		this->mutex = NULL;
+	}
+
+	/**
+	 * Begin the destruction of this packet writer. It can happen in two ways:
+	 * in the first case the client disconnected while saving the map. In this
+	 * case the saving has not finished and killed this PacketWriter. In that
+	 * case we simply set cs to NULL, triggering the appending to fail due to
+	 * the connection problem and eventually triggering the destructor. In the
+	 * second case the destructor is already called, and it is waiting for our
+	 * signal which we will send. Only then the packets will be removed by the
+	 * destructor.
+	 */
+	void Destroy()
+	{
+		if (this->mutex != NULL) this->mutex->BeginCritical();
+
+		this->cs = NULL;
+
+		if (this->mutex != NULL) this->mutex->SendSignal();
+
+		if (this->mutex != NULL) this->mutex->EndCritical();
+
+		/* Make sure the saving is completely cancelled. Yes,
+		 * we need to handle the save finish as well as the
+		 * next connection might just be requesting a map. */
+		WaitTillSaved();
+		ProcessAsyncSaveFinish();
+	}
+
+	/**
+	 * Checks whether there are packets.
+	 * It's not 100% threading safe, but this is only asked for when checking
+	 * whether there still is something to send. Then another call will be made
+	 * to actually get the Packet, which will be the only one popping packets
+	 * and thus eventually setting this on false.
+	 */
+	bool HasPackets()
+	{
+		return this->packets != NULL;
+	}
+
+	/**
+	 * Pop a single created packet from the queue with packets.
+	 */
+	Packet *PopPacket()
+	{
+		if (this->mutex != NULL) this->mutex->BeginCritical();
+
+		Packet *p = this->packets;
+		this->packets = p->next;
+		p->next = NULL;
+
+		if (this->mutex != NULL) this->mutex->EndCritical();
+
+		return p;
 	}
 
 	/** Append the current packet to the queue. */
@@ -88,7 +156,7 @@ struct PacketWriter : SaveFilter {
 	{
 		if (this->current == NULL) return;
 
-		Packet **p = &this->cs->savegame_packets;
+		Packet **p = &this->packets;
 		while (*p != NULL) {
 			p = &(*p)->next;
 		}
@@ -104,7 +172,7 @@ struct PacketWriter : SaveFilter {
 
 		if (this->current == NULL) this->current = new Packet(PACKET_SERVER_MAP_DATA);
 
-		if (this->cs->savegame_mutex != NULL) this->cs->savegame_mutex->BeginCritical();
+		if (this->mutex != NULL) this->mutex->BeginCritical();
 
 		byte *bufe = buf + size;
 		while (buf != bufe) {
@@ -119,7 +187,7 @@ struct PacketWriter : SaveFilter {
 			}
 		}
 
-		if (this->cs->savegame_mutex != NULL) this->cs->savegame_mutex->EndCritical();
+		if (this->mutex != NULL) this->mutex->EndCritical();
 
 		this->total_size += size;
 	}
@@ -129,7 +197,7 @@ struct PacketWriter : SaveFilter {
 		/* We want to abort the saving when the socket is closed. */
 		if (this->cs == NULL) SlError(STR_NETWORK_ERROR_LOSTCONNECTION);
 
-		if (this->cs->savegame_mutex != NULL) this->cs->savegame_mutex->BeginCritical();
+		if (this->mutex != NULL) this->mutex->BeginCritical();
 
 		/* Make sure the last packet is flushed. */
 		this->AppendQueue();
@@ -143,7 +211,7 @@ struct PacketWriter : SaveFilter {
 		p->Send_uint32((uint32)this->total_size);
 		this->cs->NetworkTCPSocketHandler::SendPacket(p);
 
-		if (this->cs->savegame_mutex != NULL) this->cs->savegame_mutex->EndCritical();
+		if (this->mutex != NULL) this->mutex->EndCritical();
 	}
 };
 
@@ -172,24 +240,10 @@ ServerNetworkGameSocketHandler::~ServerNetworkGameSocketHandler()
 	if (_redirect_console_to_client == this->client_id) _redirect_console_to_client = INVALID_CLIENT_ID;
 	OrderBackup::ResetUser(this->client_id);
 
-	if (this->savegame_mutex != NULL) this->savegame_mutex->BeginCritical();
-	if (this->savegame != NULL) this->savegame->cs = NULL;
-	if (this->savegame_mutex != NULL) this->savegame_mutex->EndCritical();
-
-	/* Make sure the saving is completely cancelled.
-	 * Yes, we need to handle the save finish as well
-	 * as the next connection in this "loop" might
-	 * just be requesting the map and such. */
-	WaitTillSaved();
-	ProcessAsyncSaveFinish();
-
-	while (this->savegame_packets != NULL) {
-		Packet *p = this->savegame_packets->next;
-		delete this->savegame_packets;
-		this->savegame_packets = p;
+	if (this->savegame != NULL) {
+		this->savegame->Destroy();
+		this->savegame = NULL;
 	}
-
-	delete this->savegame_mutex;
 }
 
 Packet *ServerNetworkGameSocketHandler::ReceivePacket()
@@ -205,13 +259,6 @@ Packet *ServerNetworkGameSocketHandler::ReceivePacket()
 	return p;
 }
 
-void ServerNetworkGameSocketHandler::SendPacket(Packet *packet)
-{
-	if (this->savegame_mutex != NULL) this->savegame_mutex->BeginCritical();
-	this->NetworkTCPSocketHandler::SendPacket(packet);
-	if (this->savegame_mutex != NULL) this->savegame_mutex->EndCritical();
-}
-
 NetworkRecvStatus ServerNetworkGameSocketHandler::CloseConnection(NetworkRecvStatus status)
 {
 	assert(status != NETWORK_RECV_STATUS_OKAY);
@@ -229,7 +276,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::CloseConnection(NetworkRecvSta
 		char client_name[NETWORK_CLIENT_NAME_LENGTH];
 		NetworkClientSocket *new_cs;
 
-		this->GetClientName(client_name, sizeof(client_name));
+		this->GetClientName(client_name, lastof(client_name));
 
 		NetworkTextMessage(NETWORK_ACTION_LEAVE, CC_DEFAULT, false, client_name, NULL, STR_NETWORK_ERROR_CLIENT_CONNECTION_LOST);
 
@@ -335,7 +382,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::SendCompanyInfo()
 	FOR_ALL_CLIENT_SOCKETS(csi) {
 		char client_name[NETWORK_CLIENT_NAME_LENGTH];
 
-		((ServerNetworkGameSocketHandler*)csi)->GetClientName(client_name, sizeof(client_name));
+		((ServerNetworkGameSocketHandler*)csi)->GetClientName(client_name, lastof(client_name));
 
 		ci = csi->GetInfo();
 		if (ci != NULL && Company::IsValidID(ci->client_playas)) {
@@ -397,7 +444,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::SendError(NetworkErrorCode err
 		NetworkClientSocket *new_cs;
 		char client_name[NETWORK_CLIENT_NAME_LENGTH];
 
-		this->GetClientName(client_name, sizeof(client_name));
+		this->GetClientName(client_name, lastof(client_name));
 
 		DEBUG(net, 1, "'%s' made an error and has been disconnected. Reason: '%s'", client_name, str);
 
@@ -528,7 +575,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::SendWait()
 /** This sends the map to the client */
 NetworkRecvStatus ServerNetworkGameSocketHandler::SendMap()
 {
-	static uint sent_packets; // How many packets we did send succecfully last time
+	static uint sent_packets; // How many packets we did send successfully last time
 
 	if (this->status < STATUS_AUTHORIZED) {
 		/* Illegal call, return error and ignore the packet */
@@ -556,18 +603,14 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::SendMap()
 	}
 
 	if (this->status == STATUS_MAP) {
-		if (this->savegame_mutex != NULL) this->savegame_mutex->BeginCritical();
-
 		bool last_packet = false;
+		bool has_packets = false;
 
-		for (uint i = 0; i < sent_packets && this->savegame_packets != NULL; i++) {
-			Packet *p = this->savegame_packets;
+		for (uint i = 0; (has_packets = this->savegame->HasPackets()) && i < sent_packets; i++) {
+			Packet *p = this->savegame->PopPacket();
 			last_packet = p->buffer[2] == PACKET_SERVER_MAP_DONE;
 
-			/* Remove the packet from the savegame queue and put it in the real queue. */
-			this->savegame_packets = p->next;
-			p->next = NULL;
-			this->NetworkTCPSocketHandler::SendPacket(p);
+			this->SendPacket(p);
 
 			if (last_packet) {
 				/* There is no more data, so break the for */
@@ -575,11 +618,10 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::SendMap()
 			}
 		}
 
-		if (this->savegame_mutex != NULL) this->savegame_mutex->EndCritical();
-
 		if (last_packet) {
 			/* Done reading, make sure saving is done as well */
-			WaitTillSaved();
+			this->savegame->Destroy();
+			this->savegame = NULL;
 
 			/* Set the status to DONE_MAP, no we will wait for the client
 			 *  to send it is ready (maybe that happens like never ;)) */
@@ -615,7 +657,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::SendMap()
 
 			case SPS_ALL_SENT:
 				/* All are sent, increase the sent_packets */
-				if (this->savegame_packets != NULL) sent_packets *= 2;
+				if (has_packets) sent_packets *= 2;
 				break;
 
 			case SPS_PARTLY_SENT:
@@ -899,7 +941,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::Receive_CLIENT_JOIN(Packet *p)
 	/* We need a valid name.. make it Player */
 	if (StrEmpty(name)) strecpy(name, "Player", lastof(name));
 
-	if (!NetworkFindName(name)) { // Change name if duplicate
+	if (!NetworkFindName(name, lastof(name))) { // Change name if duplicate
 		/* We could not create a name for this client */
 		return this->SendError(NETWORK_ERROR_NAME_IN_USE);
 	}
@@ -911,7 +953,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::Receive_CLIENT_JOIN(Packet *p)
 	strecpy(ci->client_name, name, lastof(ci->client_name));
 	ci->client_playas = playas;
 	ci->client_lang = client_lang;
-	DEBUG(desync, 1, "client: %08x; %02x; %02x; %04x", _date, _date_fract, (int)ci->client_playas, ci->index);
+	DEBUG(desync, 1, "client: %08x; %02x; %02x; %02x", _date, _date_fract, (int)ci->client_playas, (int)ci->index);
 
 	/* Make sure companies to which people try to join are not autocleaned */
 	if (Company::IsValidID(playas)) _network_company_states[playas].months_empty = 0;
@@ -1002,7 +1044,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::Receive_CLIENT_MAP_OK(Packet *
 		char client_name[NETWORK_CLIENT_NAME_LENGTH];
 		NetworkClientSocket *new_cs;
 
-		this->GetClientName(client_name, sizeof(client_name));
+		this->GetClientName(client_name, lastof(client_name));
 
 		NetworkTextMessage(NETWORK_ACTION_JOIN, CC_DEFAULT, false, client_name, NULL, this->client_id);
 
@@ -1073,7 +1115,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::Receive_CLIENT_COMMAND(Packet
 	}
 
 	if ((GetCommandFlags(cp.cmd) & CMD_SPECTATOR) == 0 && !Company::IsValidID(cp.company) && ci->client_id != CLIENT_ID_SERVER) {
-		IConsolePrintF(CC_ERROR, "WARNING: spectator issueing command from client %d (IP: %s), kicking...", ci->client_id, this->GetClientIP());
+		IConsolePrintF(CC_ERROR, "WARNING: spectator issuing command from client %d (IP: %s), kicking...", ci->client_id, this->GetClientIP());
 		return this->SendError(NETWORK_ERROR_KICKED);
 	}
 
@@ -1120,7 +1162,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::Receive_CLIENT_ERROR(Packet *p
 		return this->CloseConnection(NETWORK_RECV_STATUS_CONN_LOST);
 	}
 
-	this->GetClientName(client_name, sizeof(client_name));
+	this->GetClientName(client_name, lastof(client_name));
 
 	StringID strid = GetNetworkErrorMsg(errorno);
 	GetString(str, strid, lastof(str));
@@ -1152,7 +1194,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::Receive_CLIENT_QUIT(Packet *p)
 		return this->CloseConnection(NETWORK_RECV_STATUS_CONN_LOST);
 	}
 
-	this->GetClientName(client_name, sizeof(client_name));
+	this->GetClientName(client_name, lastof(client_name));
 
 	NetworkTextMessage(NETWORK_ACTION_LEAVE, CC_DEFAULT, false, client_name, NULL, STR_NETWORK_MESSAGE_CLIENT_LEAVING);
 
@@ -1319,7 +1361,8 @@ void NetworkServerSendChat(NetworkAction action, DestType desttype, int dest, co
 		}
 		default:
 			DEBUG(net, 0, "[server] received unknown chat destination type %d. Doing broadcast instead", desttype);
-			/* FALL THROUGH */
+			FALLTHROUGH;
+
 		case DESTTYPE_BROADCAST:
 			FOR_ALL_CLIENT_SOCKETS(cs) {
 				cs->SendChat(action, from_id, false, msg, data);
@@ -1354,7 +1397,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::Receive_CLIENT_CHAT(Packet *p)
 	switch (action) {
 		case NETWORK_ACTION_GIVE_MONEY:
 			if (!Company::IsValidID(ci->client_playas)) break;
-			/* FALL THROUGH */
+			FALLTHROUGH;
 		case NETWORK_ACTION_CHAT:
 		case NETWORK_ACTION_CHAT_CLIENT:
 		case NETWORK_ACTION_CHAT_COMPANY:
@@ -1401,7 +1444,7 @@ NetworkRecvStatus ServerNetworkGameSocketHandler::Receive_CLIENT_SET_NAME(Packet
 
 	if (ci != NULL) {
 		/* Display change */
-		if (NetworkFindName(client_name)) {
+		if (NetworkFindName(client_name, lastof(client_name))) {
 			NetworkTextMessage(NETWORK_ACTION_NAME_CHANGE, CC_DEFAULT, false, ci->client_name, client_name);
 			strecpy(ci->client_name, client_name, lastof(ci->client_name));
 			NetworkUpdateClientInfo(ci->client_id);
@@ -1500,7 +1543,7 @@ void NetworkSocketHandler::SendCompanyInformation(Packet *p, const Company *c, c
 	p->Send_uint64(income);
 	p->Send_uint16(c->old_economy[0].performance_history);
 
-	/* Send 1 if there is a passord for the company else send 0 */
+	/* Send 1 if there is a password for the company else send 0 */
 	p->Send_bool  (!StrEmpty(_network_company_states[c->index].password));
 
 	for (uint i = 0; i < NETWORK_VEH_END; i++) {
@@ -1620,7 +1663,7 @@ static void NetworkAutoCleanCompanies()
 		}
 	}
 
-	/* Go through all the comapnies */
+	/* Go through all the companies */
 	FOR_ALL_COMPANIES(c) {
 		/* Skip the non-active once */
 		if (c->is_ai) continue;
@@ -1632,7 +1675,7 @@ static void NetworkAutoCleanCompanies()
 			/* Is the company empty for autoclean_unprotected-months, and is there no protection? */
 			if (_settings_client.network.autoclean_unprotected != 0 && _network_company_states[c->index].months_empty > _settings_client.network.autoclean_unprotected && StrEmpty(_network_company_states[c->index].password)) {
 				/* Shut the company down */
-				DoCommandP(0, 2 | c->index << 16, CRR_AUTOCLEAN, CMD_COMPANY_CTRL);
+				DoCommandP(0, CCA_DELETE | c->index << 16 | CRR_AUTOCLEAN << 24, 0, CMD_COMPANY_CTRL);
 				IConsolePrintF(CC_DEFAULT, "Auto-cleaned company #%d with no password", c->index + 1);
 			}
 			/* Is the company empty for autoclean_protected-months, and there is a protection? */
@@ -1646,7 +1689,7 @@ static void NetworkAutoCleanCompanies()
 			/* Is the company empty for autoclean_novehicles-months, and has no vehicles? */
 			if (_settings_client.network.autoclean_novehicles != 0 && _network_company_states[c->index].months_empty > _settings_client.network.autoclean_novehicles && vehicles_in_company[c->index] == 0) {
 				/* Shut the company down */
-				DoCommandP(0, 2 | c->index << 16, CRR_AUTOCLEAN, CMD_COMPANY_CTRL);
+				DoCommandP(0, CCA_DELETE | c->index << 16 | CRR_AUTOCLEAN << 24, 0, CMD_COMPANY_CTRL);
 				IConsolePrintF(CC_DEFAULT, "Auto-cleaned company #%d with no vehicles", c->index + 1);
 			}
 		} else {
@@ -1659,16 +1702,16 @@ static void NetworkAutoCleanCompanies()
 /**
  * Check whether a name is unique, and otherwise try to make it unique.
  * @param new_name The name to check/modify.
+ * @param last     The last writeable element of the buffer.
  * @return True if an unique name was achieved.
  */
-bool NetworkFindName(char new_name[NETWORK_CLIENT_NAME_LENGTH])
+bool NetworkFindName(char *new_name, const char *last)
 {
 	bool found_name = false;
 	uint number = 0;
 	char original_name[NETWORK_CLIENT_NAME_LENGTH];
 
-	/* We use NETWORK_CLIENT_NAME_LENGTH in here, because new_name is really a pointer */
-	ttd_strlcpy(original_name, new_name, NETWORK_CLIENT_NAME_LENGTH);
+	strecpy(original_name, new_name, lastof(original_name));
 
 	while (!found_name) {
 		const NetworkClientInfo *ci;
@@ -1692,7 +1735,7 @@ bool NetworkFindName(char new_name[NETWORK_CLIENT_NAME_LENGTH])
 
 			/* Something's really wrong when there're more names than clients */
 			if (number++ > MAX_CLIENTS) break;
-			snprintf(new_name, NETWORK_CLIENT_NAME_LENGTH, "%s #%d", original_name, number);
+			seprintf(new_name, last, "%s #%d", original_name, number);
 		}
 	}
 
@@ -1786,7 +1829,7 @@ void NetworkServer_Tick(bool send_frame)
 		switch (cs->status) {
 			case NetworkClientSocket::STATUS_ACTIVE:
 				if (lag > _settings_client.network.max_lag_time) {
-					/* Client did still not report in within the specififed limit. */
+					/* Client did still not report in within the specified limit. */
 					IConsolePrintF(CC_ERROR, cs->last_packet + lag * MILLISECONDS_PER_TICK > _realtime_tick ?
 							/* A packet was received in the last three game days, so the client is likely lagging behind. */
 								"Client #%d is dropped because the client's game state is more than %d ticks behind" :
@@ -2052,7 +2095,16 @@ uint NetworkServerKickOrBanIP(ClientID client_id, bool ban)
 uint NetworkServerKickOrBanIP(const char *ip, bool ban)
 {
 	/* Add address to ban-list */
-	if (ban) *_network_ban_list.Append() = strdup(ip);
+	if (ban) {
+		bool contains = false;
+		for (char **iter = _network_ban_list.Begin(); iter != _network_ban_list.End(); iter++) {
+			if (strcmp(*iter, ip) == 0) {
+				contains = true;
+				break;
+			}
+		}
+		if (!contains) *_network_ban_list.Append() = stredup(ip);
+	}
 
 	uint n = 0;
 
@@ -2085,18 +2137,18 @@ bool NetworkCompanyHasClients(CompanyID company)
 
 
 /**
- * Get the name of the client, if the user did not send it yet, Client #<no> is used.
+ * Get the name of the client, if the user did not send it yet, Client ID is used.
  * @param client_name The variable to write the name to.
- * @param size        The amount of bytes we can write.
+ * @param last        The pointer to the last element of the destination buffer
  */
-void ServerNetworkGameSocketHandler::GetClientName(char *client_name, size_t size) const
+void ServerNetworkGameSocketHandler::GetClientName(char *client_name, const char *last) const
 {
 	const NetworkClientInfo *ci = this->GetInfo();
 
 	if (ci == NULL || StrEmpty(ci->client_name)) {
-		snprintf(client_name, size, "Client #%4d", this->client_id);
+		seprintf(client_name, last, "Client #%4d", this->client_id);
 	} else {
-		ttd_strlcpy(client_name, ci->client_name, size);
+		strecpy(client_name, ci->client_name, last);
 	}
 }
 
@@ -2122,4 +2174,37 @@ void NetworkPrintClients()
 	}
 }
 
+/**
+ * Perform all the server specific administration of a new company.
+ * @param c  The newly created company; can't be NULL.
+ * @param ci The client information of the client that made the company; can be NULL.
+ */
+void NetworkServerNewCompany(const Company *c, NetworkClientInfo *ci)
+{
+	assert(c != NULL);
+
+	if (!_network_server) return;
+
+	_network_company_states[c->index].months_empty = 0;
+	_network_company_states[c->index].password[0] = '\0';
+	NetworkServerUpdateCompanyPassworded(c->index, false);
+
+	if (ci != NULL) {
+		/* ci is NULL when replaying, or for AIs. In neither case there is a client. */
+		ci->client_playas = c->index;
+		NetworkUpdateClientInfo(ci->client_id);
+		NetworkSendCommand(0, 0, 0, CMD_RENAME_PRESIDENT, NULL, ci->client_name, c->index);
+	}
+
+	/* Announce new company on network. */
+	NetworkAdminCompanyInfo(c, true);
+
+	if (ci != NULL) {
+		/* ci is NULL when replaying, or for AIs. In neither case there is a client.
+		   We need to send Admin port update here so that they first know about the new company
+		   and then learn about a possibly joining client (see FS#6025) */
+		NetworkServerSendChat(NETWORK_ACTION_COMPANY_NEW, DESTTYPE_BROADCAST, 0, "", ci->client_id, c->index + 1);
+	}
+}
+
 #endif /* ENABLE_NETWORK */
