@@ -1,4 +1,4 @@
-/* $Id: newgrf_gui.cpp 27729 2017-01-14 13:12:49Z frosch $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -30,6 +30,7 @@
 #include "textfile_gui.h"
 #include "tilehighlight_func.h"
 #include "fios.h"
+#include "guitimer_func.h"
 
 #include "widgets/newgrf_widget.h"
 #include "widgets/misc_widget.h"
@@ -112,15 +113,15 @@ static void ShowNewGRFInfo(const GRFConfig *c, uint x, uint y, uint right, uint
 			SetDParam(0, STR_JUST_RAW_STRING);
 			SetDParamStr(1, buff);
 		} else {
-			SetDParam(0, STR_LAND_AREA_INFORMATION_LOCAL_AUTHORITY_NONE);
+			SetDParam(0, STR_NEWGRF_SETTINGS_PARAMETER_NONE);
 		}
 		y = DrawStringMultiLine(x, right, y, bottom, STR_NEWGRF_SETTINGS_PARAMETER);
 
 		/* Draw the palette of the NewGRF */
 		if (c->palette & GRFP_BLT_32BPP) {
-			SetDParamStr(0, (c->palette & GRFP_USE_WINDOWS) ? "Legacy (W) / 32 bpp" : "Default (D) / 32 bpp");
+			SetDParam(0, (c->palette & GRFP_USE_WINDOWS) ? STR_NEWGRF_SETTINGS_PALETTE_LEGACY_32BPP : STR_NEWGRF_SETTINGS_PALETTE_DEFAULT_32BPP);
 		} else {
-			SetDParamStr(0, (c->palette & GRFP_USE_WINDOWS) ? "Legacy (W)" : "Default (D)");
+			SetDParam(0, (c->palette & GRFP_USE_WINDOWS) ? STR_NEWGRF_SETTINGS_PALETTE_LEGACY : STR_NEWGRF_SETTINGS_PALETTE_DEFAULT);
 		}
 		y = DrawStringMultiLine(x, right, y, bottom, STR_NEWGRF_SETTINGS_PALETTE);
 	}
@@ -150,7 +151,7 @@ struct NewGRFParametersWindow : public Window {
 	bool clicked_increase; ///< True if the increase button was clicked, false for the decrease button.
 	bool clicked_dropdown; ///< Whether the dropdown is open.
 	bool closing_dropdown; ///< True, if the dropdown list is currently closing.
-	int timeout;           ///< How long before we unpress the last-pressed button?
+	GUITimer timeout;      ///< How long before we unpress the last-pressed button?
 	uint clicked_row;      ///< The selected parameter
 	int line_height;       ///< Height of a row in the matrix widget.
 	Scrollbar *vscroll;
@@ -162,7 +163,6 @@ struct NewGRFParametersWindow : public Window {
 		clicked_button(UINT_MAX),
 		clicked_dropdown(false),
 		closing_dropdown(false),
-		timeout(0),
 		clicked_row(UINT_MAX),
 		editable(editable)
 	{
@@ -403,7 +403,7 @@ struct NewGRFParametersWindow : public Window {
 						par_info->SetValue(this->grf_config, val);
 
 						this->clicked_button = num;
-						this->timeout = 5;
+						this->timeout.SetInterval(150);
 					}
 				} else if (par_info->type == PTYPE_UINT_ENUM && !par_info->complete_labels && click_count >= 2) {
 					/* Display a query box so users can enter a custom value. */
@@ -483,9 +483,9 @@ struct NewGRFParametersWindow : public Window {
 		}
 	}
 
-	virtual void OnTick()
+	virtual void OnRealtimeTick(uint delta_ms)
 	{
-		if (--this->timeout == 0) {
+		if (timeout.Elapsed(delta_ms)) {
 			this->clicked_button = UINT_MAX;
 			this->SetDirty();
 		}
@@ -561,7 +561,7 @@ struct NewGRFTextfileWindow : public TextfileWindow {
 
 void ShowNewGRFTextfileWindow(TextfileType file_type, const GRFConfig *c)
 {
-	DeleteWindowByClass(WC_TEXTFILE);
+	DeleteWindowById(WC_TEXTFILE, file_type);
 	new NewGRFTextfileWindow(file_type, c);
 }
 
@@ -725,7 +725,11 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 			GRFConfig *d = new GRFConfig(*a);
 			d->next = (*c)->next;
 			d->CopyParams(**c);
-			if (this->active_sel == *c) this->active_sel = NULL;
+			if (this->active_sel == *c) {
+				DeleteWindowByClass(WC_GRF_PARAMETERS);
+				DeleteWindowByClass(WC_TEXTFILE);
+				this->active_sel = NULL;
+			}
 			delete *c;
 			*c = d;
 			iter->second = d;
@@ -1022,7 +1026,10 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 				GRFConfig *c;
 				for (c = this->actives; c != NULL && i > 0; c = c->next, i--) {}
 
-				if (this->active_sel != c) DeleteWindowByClass(WC_GRF_PARAMETERS);
+				if (this->active_sel != c) {
+					DeleteWindowByClass(WC_GRF_PARAMETERS);
+					DeleteWindowByClass(WC_TEXTFILE);
+				}
 				this->active_sel = c;
 				this->avail_sel = NULL;
 				this->avail_pos = -1;
@@ -1032,12 +1039,14 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 					if (this->editable && this->active_sel != NULL) SetObjectToPlaceWnd(SPR_CURSOR_MOUSE, PAL_NONE, HT_DRAG, this);
 					break;
 				}
-				/* FALL THROUGH, with double click. */
+				/* With double click, continue */
+				FALLTHROUGH;
 			}
 
 			case WID_NS_REMOVE: { // Remove GRF
 				if (this->active_sel == NULL || !this->editable) break;
 				DeleteWindowByClass(WC_GRF_PARAMETERS);
+				DeleteWindowByClass(WC_TEXTFILE);
 
 				/* Choose the next GRF file to be the selected file. */
 				GRFConfig *newsel = this->active_sel->next;
@@ -1079,6 +1088,7 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 				this->active_sel = NULL;
 				DeleteWindowByClass(WC_GRF_PARAMETERS);
 				if (i < this->avails.Length()) {
+					if (this->avail_sel != this->avails[i]) DeleteWindowByClass(WC_TEXTFILE);
 					this->avail_sel = this->avails[i];
 					this->avail_pos = i;
 				}
@@ -1087,7 +1097,8 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 					if (this->editable && this->avail_sel != NULL && !HasBit(this->avail_sel->flags, GCF_INVALID)) SetObjectToPlaceWnd(SPR_CURSOR_MOUSE, PAL_NONE, HT_DRAG, this);
 					break;
 				}
-				/* FALL THROUGH, with double click. */
+				/* With double click, continue */
+				FALLTHROUGH;
 			}
 
 			case WID_NS_ADD:
@@ -1150,11 +1161,11 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 
 	virtual void OnNewGRFsScanned()
 	{
+		if (this->active_sel == NULL) DeleteWindowByClass(WC_TEXTFILE);
 		this->avail_sel = NULL;
 		this->avail_pos = -1;
 		this->avails.ForceRebuild();
 		this->DeleteChildWindows(WC_QUERY_STRING); // Remove the parameter query window
-		this->DeleteChildWindows(WC_TEXTFILE);     // Remove the view textfile window
 	}
 
 	virtual void OnDropdownSelect(int widget, int index)
@@ -1171,6 +1182,7 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 
 		ResetObjectToPlace();
 		DeleteWindowByClass(WC_GRF_PARAMETERS);
+		DeleteWindowByClass(WC_TEXTFILE);
 		this->active_sel = NULL;
 		this->InvalidateData(GOID_NEWGRF_PRESET_LOADED);
 	}
@@ -1219,16 +1231,18 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 					*l = new GRFConfig(*f);
 					(*l)->next = c->next;
 
-					if (active_sel == c) active_sel = *l;
+					if (this->active_sel == c) this->active_sel = *l;
 
 					delete c;
 				}
 
 				this->avails.ForceRebuild();
-				/* FALL THROUGH */
+				FALLTHROUGH;
+
 			case GOID_NEWGRF_LIST_EDITED:
 				this->preset = -1;
-				/* FALL THROUGH */
+				FALLTHROUGH;
+
 			case GOID_NEWGRF_PRESET_LOADED: {
 				/* Update scrollbars */
 				int i = 0;
@@ -1343,6 +1357,9 @@ struct NewGRFWindow : public Window, NewGRFScanCallback {
 
 		if (this->avails.Length() == 0) this->avail_pos = -1;
 		if (this->avail_pos >= 0) {
+			this->active_sel = NULL;
+			DeleteWindowByClass(WC_GRF_PARAMETERS);
+			if (this->avail_sel != this->avails[this->avail_pos]) DeleteWindowByClass(WC_TEXTFILE);
 			this->avail_sel = this->avails[this->avail_pos];
 			this->vscroll2->ScrollTowards(this->avail_pos);
 			this->InvalidateData(0);
@@ -1505,6 +1522,8 @@ private:
 	{
 		if (this->avail_sel == NULL || !this->editable || HasBit(this->avail_sel->flags, GCF_INVALID)) return false;
 
+		DeleteWindowByClass(WC_TEXTFILE);
+
 		uint count = 0;
 		GRFConfig **entry = NULL;
 		GRFConfig **list;
@@ -1580,8 +1599,8 @@ NewGRFWindow::GUIGRFConfigList::FilterFunction * const NewGRFWindow::filter_func
 /**
  * Custom nested widget container for the NewGRF gui.
  * Depending on the space in the gui, it uses either
- * - two column mode, put the #acs and the #avs underneath each other and the #info next to it, or
- * - three column mode, put the #avs, #acs, and #info each in its own column.
+ * - two column mode, put the #acs and the #avs underneath each other and the #inf next to it, or
+ * - three column mode, put the #avs, #acs, and #inf each in its own column.
  */
 class NWidgetNewGRFDisplay : public NWidgetContainer {
 public:
@@ -1964,6 +1983,7 @@ static void NewGRFConfirmationCallback(Window *w, bool confirmed)
 {
 	if (confirmed) {
 		DeleteWindowByClass(WC_GRF_PARAMETERS);
+		DeleteWindowByClass(WC_TEXTFILE);
 		NewGRFWindow *nw = dynamic_cast<NewGRFWindow*>(w);
 
 		GamelogStartAction(GLAT_GRF);
