@@ -1,4 +1,4 @@
-/* $Id: newgrf_station.cpp 27984 2018-03-11 13:19:41Z frosch $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -881,7 +881,7 @@ bool CanStationTileHaveWires(TileIndex tile)
 	return statspec == NULL || !HasBit(statspec->wires, GetStationGfx(tile));
 }
 
-/** Wrapper for animation control, see #GetStationCallback. */
+/** Wrapper for animation control, see GetStationCallback. */
 uint16 GetAnimStationCallback(CallbackID callback, uint32 param1, uint32 param2, const StationSpec *statspec, BaseStation *st, TileIndex tile, int extra_data)
 {
 	return GetStationCallback(callback, param1, param2, statspec, st, tile);
@@ -963,7 +963,7 @@ void TriggerStationRandomisation(Station *st, TileIndex tile, StationRandomTrigg
 	uint32 whole_reseed = 0;
 	ETileArea area = ETileArea(st, tile, tas[trigger]);
 
-	uint32 empty_mask = 0;
+	CargoTypes empty_mask = 0;
 	if (trigger == SRT_CARGO_TAKEN) {
 		/* Create a bitmask of completely empty cargo types to be matched */
 		for (CargoID i = 0; i < NUM_CARGO; i++) {
