@@ -45,8 +45,8 @@ const NewsItem *_statusbar_news_item = NULL;
 
 static uint MIN_NEWS_AMOUNT = 30;      ///< preferred minimum amount of news messages
 static uint _total_news = 0;           ///< current number of news items
-NewsItem *_oldest_news = NULL;         ///< head of news items queue
-static NewsItem *_latest_news = NULL;  ///< tail of news items queue
+static NewsItem *_oldest_news = NULL;  ///< head of news items queue
+NewsItem *_latest_news = NULL;         ///< tail of news items queue
 
 /**
  * Forced news item.
@@ -54,7 +54,7 @@ static NewsItem *_latest_news = NULL;  ///< tail of news items queue
  * If the message being shown was forced by the user, a pointer is stored
  * in _forced_news. Otherwise, \a _forced_news variable is NULL.
  */
-static const NewsItem *_forced_news = NULL;       ///< item the user has asked for
+static const NewsItem *_forced_news = NULL;
 
 /** Current news item (last item shown regularly). */
 static const NewsItem *_current_news = NULL;
@@ -331,6 +331,11 @@ struct NewsWindow : Window {
 				*size = maxdim(*size, GetSpriteSize(SPR_GRADIENT));
 				break;
 
+			case WID_N_MGR_NAME:
+				SetDParamStr(0, static_cast<const CompanyNewsInformation *>(this->ni->free_data)->president_name);
+				str = STR_JUST_RAW_STRING;
+				break;
+
 			case WID_N_MESSAGE:
 				CopyInDParam(0, this->ni->params, lengthof(this->ni->params));
 				str = this->ni->string_id;
