@@ -1,4 +1,4 @@
-/* $Id: yapf_costrail.hpp 27864 2017-05-03 20:13:05Z frosch $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -104,7 +104,7 @@ public:
 		assert(IsValidTrackdir(td2));
 		int cost = 0;
 		if (TrackFollower::Allow90degTurns()
-				&& ((TrackdirToTrackdirBits(td2) & (TrackdirBits)TrackdirCrossesTrackdirs(td1)) != 0)) {
+				&& HasTrackdir(TrackdirCrossesTrackdirs(td1), td2)) {
 			/* 90-deg curve penalty */
 			cost += Yapf().PfGetSettings().rail_curve90_penalty;
 		} else if (td2 != NextTrackdir(td1)) {
@@ -280,7 +280,7 @@ public:
 	{
 		assert(!n.flags_u.flags_s.m_targed_seen);
 		assert(tf->m_new_tile == n.m_key.m_tile);
-		assert((TrackdirToTrackdirBits(n.m_key.m_td) & tf->m_new_td_bits) != TRACKDIR_BIT_NONE);
+		assert((HasTrackdir(tf->m_new_td_bits, n.m_key.m_td)));
 
 		CPerfStart perf_cost(Yapf().m_perf_cost);
 
@@ -562,6 +562,9 @@ no_entry_cost: // jump here at the beginning if the node has no parent (it is th
 
 		} // for (;;)
 
+		/* Don't consider path any further it if exceeded max_cost. */
+		if (end_segment_reason & ESRB_PATH_TOO_LONG) return false;
+
 		bool target_seen = false;
 		if ((end_segment_reason & ESRB_POSSIBLE_TARGET) != ESRB_NONE) {
 			/* Depot, station or waypoint. */
