@@ -1,4 +1,4 @@
-/* $Id: script_engine.hpp 24606 2012-10-17 19:11:03Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,6 +15,7 @@
 #include "script_vehicle.hpp"
 #include "script_rail.hpp"
 #include "script_airport.hpp"
+#include "script_date.hpp"
 
 /**
  * Class that handles all engine related functions.
@@ -23,15 +24,17 @@
 class ScriptEngine : public ScriptObject {
 public:
 	/**
-	 * Checks whether the given engine type is valid. An engine is valid if you
-	 * have at least one vehicle of this engine or it's currently buildable.
+	 * Checks whether the given engine type is valid.
+	 * An engine is valid for a company if it has at least one vehicle of this engine or it's currently buildable.
+	 * @game Outside ScriptCompanyMode scope the function reports all engines valid, which were or will be available at some point.
 	 * @param engine_id The engine to check.
 	 * @return True if and only if the engine type is valid.
 	 */
 	static bool IsValidEngine(EngineID engine_id);
 
 	/**
-	 * Checks whether the given engine type is buildable by you.
+	 * Checks whether the given engine type is buildable for a company.
+	 * @game Outside ScriptCompanyMode scope the function checks whether the engine is currently buildable by all companies (no exclusive preview).
 	 * @param engine_id The engine to check.
 	 * @return True if and only if the engine type is buildable.
 	 */
@@ -105,7 +108,6 @@ public:
 	 * Get the maximum speed of an engine.
 	 * @param engine_id The engine to get the maximum speed of.
 	 * @pre IsValidEngine(engine_id).
-	 * @pre GetVehicleType(engine_id) != ScriptVehicle::VT_TRAIN || !IsWagon(engine_id).
 	 * @return The maximum speed the engine has.
 	 * @note The speed is in OpenTTD's internal speed unit.
 	 *       This is mph / 1.6, which is roughly km/h.
@@ -172,7 +174,7 @@ public:
 	 * @pre IsValidEngine(engine_id).
 	 * @return The date this engine was designed.
 	 */
-	static int32 GetDesignDate(EngineID engine_id);
+	static ScriptDate::Date GetDesignDate(EngineID engine_id);
 
 	/**
 	 * Get the type of an engine.
@@ -253,13 +255,15 @@ public:
 
 	/**
 	 * Get the maximum allowed distance between two orders for an engine.
-	 * The distance returned is a vehicle-type specific distance indepenent from other
+	 * The distance returned is a vehicle-type specific distance independent from other
 	 * map distances, you may use the result of this function to compare it
 	 * with the result of ScriptOrder::GetOrderDistance.
 	 * @param engine_id The engine to get the max distance for.
 	 * @pre IsValidEngine(engine_id).
 	 * @return The maximum distance between two orders for the engine
-	 *  or 0 if the distance is unlimited.
+	 *         or 0 if the distance is unlimited.
+	 * @note   The unit of the order distances is unspecified and should
+	 *         not be compared with map distances
 	 * @see ScriptOrder::GetOrderDistance
 	 */
 	static uint GetMaximumOrderDistance(EngineID engine_id);
