@@ -1,4 +1,4 @@
-/* $Id: script_rail.hpp 23633 2011-12-19 21:05:36Z truebrain $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -32,8 +32,8 @@ public:
 		/** One-way roads cannot have crossings */
 		ERR_CROSSING_ON_ONEWAY_ROAD,       // [STR_ERROR_CROSSING_ON_ONEWAY_ROAD]
 
-		/** Track not suitable for signals */
-		ERR_UNSUITABLE_TRACK,              // [STR_ERROR_NO_SUITABLE_RAILROAD_TRACK]
+		/** No suitable track could be found */
+		ERR_UNSUITABLE_TRACK,              // [STR_ERROR_NO_SUITABLE_RAILROAD_TRACK, STR_ERROR_THERE_IS_NO_RAILROAD_TRACK, STR_ERROR_THERE_ARE_NO_SIGNALS, STR_ERROR_THERE_IS_NO_STATION]
 
 		/** This railtype cannot have crossings */
 		ERR_RAILTYPE_DISALLOWS_CROSSING,   // [STR_ERROR_CROSSING_DISALLOWED]
@@ -268,6 +268,9 @@ public:
 	 * Build a NewGRF rail station. This calls callback 18 to let a NewGRF
 	 *  provide the station class / id to build, so we don't end up with
 	 *  only the default stations on the map.
+	 * When no NewGRF provides a rail station, or an unbuildable rail station is
+	 *  returned by a NewGRF, this function will fall back to building a default
+	 *  non-NewGRF station as if ScriptRail::BuildRailStation was called.
 	 * @param tile Place to build the station.
 	 * @param direction The direction to build the station.
 	 * @param num_platforms The number of platforms to build.
@@ -314,11 +317,12 @@ public:
 	/**
 	 * Remove all rail waypoint pieces within a rectangle on the map.
 	 * @param tile One corner of the rectangle to clear.
-	 * @param tile2 The oppposite corner.
+	 * @param tile2 The opposite corner.
 	 * @param keep_rail Whether to keep the rail after removal.
 	 * @pre IsValidTile(tile).
 	 * @pre IsValidTile(tile2).
 	 * @game @pre Valid ScriptCompanyMode active in scope.
+	 * @exception ScriptRail::ERR_UNSUITABLE_TRACK
 	 * @return Whether at least one tile has been/can be cleared or not.
 	 */
 	static bool RemoveRailWaypointTileRectangle(TileIndex tile, TileIndex tile2, bool keep_rail);
@@ -326,11 +330,12 @@ public:
 	/**
 	 * Remove all rail station platform pieces within a rectangle on the map.
 	 * @param tile One corner of the rectangle to clear.
-	 * @param tile2 The oppposite corner.
+	 * @param tile2 The opposite corner.
 	 * @param keep_rail Whether to keep the rail after removal.
 	 * @pre IsValidTile(tile).
 	 * @pre IsValidTile(tile2).
 	 * @game @pre Valid ScriptCompanyMode active in scope.
+	 * @exception ScriptRail::ERR_UNSUITABLE_TRACK
 	 * @return Whether at least one tile has been/can be cleared or not.
 	 */
 	static bool RemoveRailStationTileRectangle(TileIndex tile, TileIndex tile2, bool keep_rail);
@@ -369,6 +374,7 @@ public:
 	 * @pre ScriptMap::IsValidTile(tile).
 	 * @pre (GetRailTracks(tile) & rail_track) != 0.
 	 * @game @pre Valid ScriptCompanyMode active in scope.
+	 * @exception ScriptRail::ERR_UNSUITABLE_TRACK
 	 * @return Whether the rail has been/can be removed or not.
 	 * @note You can only remove a single track with this function so do not
 	 *   use the values from RailTrack as bitmask.
@@ -424,6 +430,7 @@ public:
 	 *      (ScriptMap::GetTileX(from) == ScriptMap::GetTileX(tile) && ScriptMap::GetTileX(tile) == ScriptMap::GetTileX(to)) ||
 	 *      (ScriptMap::GetTileY(from) == ScriptMap::GetTileY(tile) && ScriptMap::GetTileY(tile) == ScriptMap::GetTileY(to)).
 	 * @game @pre Valid ScriptCompanyMode active in scope.
+	 * @exception ScriptRail::ERR_UNSUITABLE_TRACK
 	 * @return Whether the rail has been/can be removed or not.
 	 */
 	static bool RemoveRail(TileIndex from, TileIndex tile, TileIndex to);
@@ -457,6 +464,7 @@ public:
 	 * @pre ScriptMap::DistanceManhattan(tile, front) == 1.
 	 * @pre GetSignalType(tile, front) != SIGNALTYPE_NONE.
 	 * @game @pre Valid ScriptCompanyMode active in scope.
+	 * @exception ScriptRail::ERR_UNSUITABLE_TRACK
 	 * @return Whether the signal has been/can be removed or not.
 	 */
 	static bool RemoveSignal(TileIndex tile, TileIndex front);
