@@ -1,4 +1,4 @@
-/* $Id: strgen.cpp 23740 2012-01-03 21:32:51Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -21,14 +21,14 @@
 #include <stdarg.h>
 #include <exception>
 
-#if (!defined(WIN32) && !defined(WIN64)) || defined(__CYGWIN__)
+#if !defined(_WIN32) || defined(__CYGWIN__)
 #include <unistd.h>
 #include <sys/stat.h>
 #endif
 
-#if defined WIN32 || defined __WATCOMC__
+#if defined(_WIN32) || defined(__WATCOMC__)
 #include <direct.h>
-#endif /* WIN32 || __WATCOMC__ */
+#endif /* _WIN32 || __WATCOMC__ */
 
 #ifdef __MORPHOS__
 #ifdef stderr
@@ -39,6 +39,8 @@
 
 #include "../table/strgen_tables.h"
 
+#include "../safeguards.h"
+
 
 #ifdef _MSC_VER
 # define LINE_NUM_FMT(s) "%s (%d): warning: %s (" s ")\n"
@@ -51,7 +53,7 @@ void CDECL strgen_warning(const char *s, ...)
 	char buf[1024];
 	va_list va;
 	va_start(va, s);
-	vsnprintf(buf, lengthof(buf), s, va);
+	vseprintf(buf, lastof(buf), s, va);
 	va_end(va);
 	fprintf(stderr, LINE_NUM_FMT("warning"), _file, _cur_line, buf);
 	_warnings++;
@@ -62,7 +64,7 @@ void CDECL strgen_error(const char *s, ...)
 	char buf[1024];
 	va_list va;
 	va_start(va, s);
-	vsnprintf(buf, lengthof(buf), s, va);
+	vseprintf(buf, lastof(buf), s, va);
 	va_end(va);
 	fprintf(stderr, LINE_NUM_FMT("error"), _file, _cur_line, buf);
 	_errors++;
@@ -73,7 +75,7 @@ void NORETURN CDECL strgen_fatal(const char *s, ...)
 	char buf[1024];
 	va_list va;
 	va_start(va, s);
-	vsnprintf(buf, lengthof(buf), s, va);
+	vseprintf(buf, lastof(buf), s, va);
 	va_end(va);
 	fprintf(stderr, LINE_NUM_FMT("FATAL"), _file, _cur_line, buf);
 #ifdef _MSC_VER
@@ -87,7 +89,7 @@ void NORETURN CDECL error(const char *s, ...)
 	char buf[1024];
 	va_list va;
 	va_start(va, s);
-	vsnprintf(buf, lengthof(buf), s, va);
+	vseprintf(buf, lastof(buf), s, va);
 	va_end(va);
 	fprintf(stderr, LINE_NUM_FMT("FATAL"), _file, _cur_line, buf);
 #ifdef _MSC_VER
@@ -120,9 +122,9 @@ struct FileStringReader : StringReader {
 		fclose(this->fh);
 	}
 
-	/* virtual */ char *ReadLine(char *buffer, size_t size)
+	/* virtual */ char *ReadLine(char *buffer, const char *last)
 	{
-		return fgets(buffer, size, this->fh);
+		return fgets(buffer, ClampToU16(last - buffer + 1), this->fh);
 	}
 
 	/* virtual */ void HandlePragma(char *str);
@@ -147,11 +149,6 @@ void FileStringReader::HandlePragma(char *str)
 		strecpy(_lang.own_name, str + 8, lastof(_lang.own_name));
 	} else if (!memcmp(str, "isocode ", 8)) {
 		strecpy(_lang.isocode, str + 8, lastof(_lang.isocode));
-	} else if (!memcmp(str, "plural ", 7)) {
-		_lang.plural_form = atoi(str + 7);
-		if (_lang.plural_form >= lengthof(_plural_forms)) {
-			error("Invalid pluralform %d", _lang.plural_form);
-		}
 	} else if (!memcmp(str, "textdir ", 8)) {
 		if (!memcmp(str + 8, "ltr", 3)) {
 			_lang.text_dir = TD_LTR;
@@ -208,7 +205,7 @@ void FileStringReader::HandlePragma(char *str)
 			_lang.num_cases++;
 		}
 	} else {
-		error("unknown pragma '%s'", str);
+		StringReader::HandlePragma(str);
 	}
 }
 
@@ -218,7 +215,10 @@ bool CompareFiles(const char *n1, const char *n2)
 	if (f2 == NULL) return false;
 
 	FILE *f1 = fopen(n1, "rb");
-	if (f1 == NULL) error("can't open %s", n1);
+	if (f1 == NULL) {
+		fclose(f2);
+		error("can't open %s", n1);
+	}
 
 	size_t l1, l2;
 	do {
@@ -250,7 +250,7 @@ struct FileWriter {
 	 */
 	FileWriter(const char *filename)
 	{
-		this->filename = strdup(filename);
+		this->filename = stredup(filename);
 		this->fh = fopen(this->filename, "wb");
 
 		if (this->fh == NULL) {
@@ -268,7 +268,7 @@ struct FileWriter {
 	/** Make sure the file is closed. */
 	virtual ~FileWriter()
 	{
-		/* If we weren't closed an exception was thrown, so remove the termporary file. */
+		/* If we weren't closed an exception was thrown, so remove the temporary file. */
 		if (fh != NULL) {
 			fclose(this->fh);
 			unlink(this->filename);
@@ -288,13 +288,19 @@ struct HeaderFileWriter : HeaderWriter, FileWriter {
 	 * @param filename The file to open.
 	 */
 	HeaderFileWriter(const char *filename) : FileWriter("tmp.xxx"),
-		real_filename(strdup(filename)), prev(0)
+		real_filename(stredup(filename)), prev(0)
 	{
 		fprintf(this->fh, "/* This file is automatically generated. Do not modify */\n\n");
 		fprintf(this->fh, "#ifndef TABLE_STRINGS_H\n");
 		fprintf(this->fh, "#define TABLE_STRINGS_H\n");
 	}
 
+	/** Free the filename. */
+	~HeaderFileWriter()
+	{
+		free(real_filename);
+	}
+
 	void WriteStringID(const char *name, int stringid)
 	{
 		if (prev + 1 != stringid) fprintf(this->fh, "\n");
@@ -327,9 +333,9 @@ struct HeaderFileWriter : HeaderWriter, FileWriter {
 			unlink(this->filename);
 		} else {
 			/* else rename tmp.xxx into filename */
-	#if defined(WIN32) || defined(WIN64)
+#	if defined(_WIN32)
 			unlink(this->real_filename);
-	#endif
+#	endif
 			if (rename(this->filename, this->real_filename) == -1) error("rename() failed");
 		}
 	}
@@ -352,7 +358,9 @@ struct LanguageFileWriter : LanguageWriter, FileWriter {
 
 	void Finalise()
 	{
-		fputc(0, this->fh);
+		if (fputc(0, this->fh) == EOF) {
+			error("Could not write to %s", this->filename);
+		}
 		this->FileWriter::Finalise();
 	}
 
@@ -367,25 +375,27 @@ struct LanguageFileWriter : LanguageWriter, FileWriter {
 /** Multi-OS mkdirectory function */
 static inline void ottd_mkdir(const char *directory)
 {
-#if defined(WIN32) || defined(__WATCOMC__)
-		mkdir(directory);
+	/* Ignore directory creation errors; they'll surface later on, and most
+	 * of the time they are 'directory already exists' errors anyhow. */
+#if defined(_WIN32) || defined(__WATCOMC__)
+	mkdir(directory);
 #else
-		mkdir(directory, 0755);
+	mkdir(directory, 0755);
 #endif
 }
 
 /**
  * Create a path consisting of an already existing path, a possible
- * path seperator and the filename. The seperator is only appended if the path
- * does not already end with a seperator
+ * path separator and the filename. The separator is only appended if the path
+ * does not already end with a separator
  */
-static inline char *mkpath(char *buf, size_t buflen, const char *path, const char *file)
+static inline char *mkpath(char *buf, const char *last, const char *path, const char *file)
 {
-	ttd_strlcpy(buf, path, buflen); // copy directory into buffer
+	strecpy(buf, path, last); // copy directory into buffer
 
-	char *p = strchr(buf, '\0'); // add path seperator if necessary
-	if (p[-1] != PATHSEPCHAR && (size_t)(p - buf) + 1 < buflen) *p++ = PATHSEPCHAR;
-	ttd_strlcpy(p, file, buflen - (size_t)(p - buf)); // catenate filename at end of buffer
+	char *p = strchr(buf, '\0'); // add path separator if necessary
+	if (p[-1] != PATHSEPCHAR && p != last) *p++ = PATHSEPCHAR;
+	strecpy(p, file, last); // concatenate filename at end of buffer
 	return buf;
 }
 
@@ -432,40 +442,30 @@ int CDECL main(int argc, char *argv[])
 
 		switch (i) {
 			case 'v':
-				puts("$Revision: 23740 $");
+				puts("$Revision$");
 				return 0;
 
 			case 'C':
 				printf("args\tflags\tcommand\treplacement\n");
 				for (const CmdStruct *cs = _cmd_structs; cs < endof(_cmd_structs); cs++) {
 					char flags;
-					switch (cs->value) {
-						case 0x200E: case 0x200F: // Implicit BIDI controls
-						case 0x202A: case 0x202B: case 0x202C: case 0x202D: case 0x202E: // Explicit BIDI controls
-						case 0xA0: // Non breaking space
-						case '\n': // Newlines may be added too
-						case '{':  // This special
-							/* This command may be in the translation when it is not in base */
-							flags = 'i';
-							break;
-
-						default:
-							if (cs->proc == EmitGender) {
-								flags = 'g'; // Command needs number of parameters defined by number of genders
-							} else if (cs->proc == EmitPlural) {
-								flags = 'p'; // Command needs number of parameters defined by plural value
-							} else {
-								flags = '0'; // Command needs no parameters
-							}
+					if (cs->proc == EmitGender) {
+						flags = 'g'; // Command needs number of parameters defined by number of genders
+					} else if (cs->proc == EmitPlural) {
+						flags = 'p'; // Command needs number of parameters defined by plural value
+					} else if (cs->flags & C_DONTCOUNT) {
+						flags = 'i'; // Command may be in the translation when it is not in base
+					} else {
+						flags = '0'; // Command needs no parameters
 					}
 					printf("%i\t%c\t\"%s\"\t\"%s\"\n", cs->consumes, flags, cs->cmd, strstr(cs->cmd, "STRING") ? "STRING" : cs->cmd);
 				}
 				return 0;
 
 			case 'L':
-				printf("count\tdescription\n");
+				printf("count\tdescription\tnames\n");
 				for (const PluralForm *pf = _plural_forms; pf < endof(_plural_forms); pf++) {
-					printf("%i\t\"%s\"\n", pf->plural_count, pf->description);
+					printf("%i\t\"%s\"\t%s\n", pf->plural_count, pf->description, pf->names);
 				}
 				return 0;
 
@@ -487,7 +487,7 @@ int CDECL main(int argc, char *argv[])
 
 			case 'h':
 				puts(
-					"strgen - $Revision: 23740 $\n"
+					"strgen - $Revision$\n"
 					" -v | --version    print version information and exit\n"
 					" -t | --todo       replace any untranslated strings with '<TODO>'\n"
 					" -w | --warning    print a warning for any untranslated strings\n"
@@ -525,27 +525,28 @@ int CDECL main(int argc, char *argv[])
 		 * with a (free) parameter the program will translate that language to destination
 		 * directory. As input english.txt is parsed from the source directory */
 		if (mgo.numleft == 0) {
-			mkpath(pathbuf, lengthof(pathbuf), src_dir, "english.txt");
+			mkpath(pathbuf, lastof(pathbuf), src_dir, "english.txt");
 
 			/* parse master file */
-			StringData data(TAB_COUNT);
+			StringData data(TEXT_TAB_END);
 			FileStringReader master_reader(data, pathbuf, true, false);
 			master_reader.ParseFile();
 			if (_errors != 0) return 1;
 
 			/* write strings.h */
 			ottd_mkdir(dest_dir);
-			mkpath(pathbuf, lengthof(pathbuf), dest_dir, "strings.h");
+			mkpath(pathbuf, lastof(pathbuf), dest_dir, "strings.h");
 
 			HeaderFileWriter writer(pathbuf);
 			writer.WriteHeader(data);
 			writer.Finalise(data);
+			if (_errors != 0) return 1;
 		} else if (mgo.numleft >= 1) {
 			char *r;
 
-			mkpath(pathbuf, lengthof(pathbuf), src_dir, "english.txt");
+			mkpath(pathbuf, lastof(pathbuf), src_dir, "english.txt");
 
-			StringData data(TAB_COUNT);
+			StringData data(TEXT_TAB_END);
 			/* parse master file and check if target file is correct */
 			FileStringReader master_reader(data, pathbuf, true, false);
 			master_reader.ParseFile();
@@ -561,12 +562,12 @@ int CDECL main(int argc, char *argv[])
 
 				/* get the targetfile, strip any directories and append to destination path */
 				r = strrchr(mgo.argv[i], PATHSEPCHAR);
-				mkpath(pathbuf, lengthof(pathbuf), dest_dir, (r != NULL) ? &r[1] : mgo.argv[i]);
+				mkpath(pathbuf, lastof(pathbuf), dest_dir, (r != NULL) ? &r[1] : mgo.argv[i]);
 
 				/* rename the .txt (input-extension) to .lng */
 				r = strrchr(pathbuf, '.');
 				if (r == NULL || strcmp(r, ".txt") != 0) r = strchr(pathbuf, '\0');
-				ttd_strlcpy(r, ".lng", (size_t)(r - pathbuf));
+				strecpy(r, ".lng", lastof(pathbuf));
 
 				LanguageFileWriter writer(pathbuf);
 				writer.WriteLang(data);
