@@ -1,4 +1,4 @@
-/* $Id: subsidy.cpp 24228 2012-05-12 07:31:25Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -24,9 +24,12 @@
 #include "core/random_func.hpp"
 #include "game/game.hpp"
 #include "command_func.h"
+#include "string_func.h"
 
 #include "table/strings.h"
 
+#include "safeguards.h"
+
 SubsidyPool _subsidy_pool("Subsidy"); ///< Pool for the subsidies.
 INSTANTIATE_POOL_METHODS(Subsidy)
 
@@ -45,7 +48,7 @@ void Subsidy::AwardTo(CompanyID company)
 	SetDParam(0, company);
 	GetString(company_name, STR_COMPANY_NAME, lastof(company_name));
 
-	char *cn = strdup(company_name);
+	char *cn = stredup(company_name);
 
 	/* Add a news item */
 	Pair reftype = SetupSubsidyDecodeParam(this, false);
@@ -54,7 +57,7 @@ void Subsidy::AwardTo(CompanyID company)
 	SetDParamStr(0, cn);
 	AddNewsItem(
 		STR_NEWS_SERVICE_SUBSIDY_AWARDED_HALF + _settings_game.difficulty.subsidy_multiplier,
-		NS_SUBSIDIES,
+		NT_SUBSIDIES, NF_NORMAL,
 		(NewsReferenceType)reftype.a, this->src, (NewsReferenceType)reftype.b, this->dst,
 		cn
 	);
@@ -189,9 +192,9 @@ static bool CheckSubsidyDuplicate(CargoID cargo, SourceType src_type, SourceID s
 
 /**
  * Checks if the source and destination of a subsidy are inside the distance limit.
- * @param src_type Type of #src.
+ * @param src_type Type of \a src.
  * @param src      Index of source.
- * @param dst_type Type of #dst.
+ * @param dst_type Type of \a dst.
  * @param dst      Index of destination.
  * @return True if they are inside the distance limit.
  */
@@ -206,9 +209,9 @@ static bool CheckSubsidyDistance(SourceType src_type, SourceID src, SourceType d
 /**
  * Creates a subsidy with the given parameters.
  * @param cid      Subsidised cargo.
- * @param src_type Type of #src.
+ * @param src_type Type of \a src.
  * @param src      Index of source.
- * @param dst_type Type of #dst.
+ * @param dst_type Type of \a dst.
  * @param dst      Index of destination.
  */
 void CreateSubsidy(CargoID cid, SourceType src_type, SourceID src, SourceType dst_type, SourceID dst)
@@ -223,7 +226,7 @@ void CreateSubsidy(CargoID cid, SourceType src_type, SourceID src, SourceType ds
 	s->awarded = INVALID_COMPANY;
 
 	Pair reftype = SetupSubsidyDecodeParam(s, false);
-	AddNewsItem(STR_NEWS_SERVICE_SUBSIDY_OFFERED, NS_SUBSIDIES, (NewsReferenceType)reftype.a, s->src, (NewsReferenceType)reftype.b, s->dst);
+	AddNewsItem(STR_NEWS_SERVICE_SUBSIDY_OFFERED, NT_SUBSIDIES, NF_NORMAL, (NewsReferenceType)reftype.a, s->src, (NewsReferenceType)reftype.b, s->dst);
 	SetPartOfSubsidyFlag(s->src_type, s->src, POS_SRC);
 	SetPartOfSubsidyFlag(s->dst_type, s->dst, POS_DST);
 	AI::BroadcastNewEvent(new ScriptEventSubsidyOffer(s->index));
@@ -242,7 +245,7 @@ void CreateSubsidy(CargoID cid, SourceType src_type, SourceID src, SourceType ds
  * - p1 = (bit 24 - 31) - CargoID of subsidy.
  * @param p2 various bitstuffed elements
  * - p2 = (bit  0 -  7) - SourceType of destination.
- * - p2 = (bit  8 - 23) - SourceID of destionation.
+ * - p2 = (bit  8 - 23) - SourceID of destination.
  * @param text unused.
  * @return the cost of this operation or an error
  */
@@ -330,8 +333,9 @@ bool FindSubsidyTownCargoRoute()
 
 	/* Select a random town. */
 	const Town *src_town = Town::GetRandom();
+	if (src_town->cache.population < SUBSIDY_CARGO_MIN_POPULATION) return false;
 
-	uint32 town_cargo_produced = src_town->cargo_produced;
+	CargoTypes town_cargo_produced = src_town->cargo_produced;
 
 	/* Passenger subsidies are not handled here. */
 	ClrBit(town_cargo_produced, CT_PASSENGERS);
@@ -348,7 +352,10 @@ bool FindSubsidyTownCargoRoute()
 	}
 
 	/* Avoid using invalid NewGRF cargoes. */
-	if (!CargoSpec::Get(cid)->IsValid()) return false;
+	if (!CargoSpec::Get(cid)->IsValid() ||
+			_settings_game.linkgraph.GetDistributionType(cid) != DT_MANUAL) {
+		return false;
+	}
 
 	/* Quit if the percentage transported is large enough. */
 	if (src_town->GetPercentTransported(cid) > SUBSIDY_MAX_PCT_TRANSPORTED) return false;
@@ -377,19 +384,30 @@ bool FindSubsidyIndustryCargoRoute()
 	CargoID cid;
 
 	/* Randomize cargo type */
-	if (src_ind->produced_cargo[1] != CT_INVALID && HasBit(Random(), 0)) {
-		cid = src_ind->produced_cargo[1];
-		trans = src_ind->last_month_pct_transported[1];
-		total = src_ind->last_month_production[1];
-	} else {
-		cid = src_ind->produced_cargo[0];
-		trans = src_ind->last_month_pct_transported[0];
-		total = src_ind->last_month_production[0];
+	int num_cargos = 0;
+	uint cargo_index;
+	for (cargo_index = 0; cargo_index < lengthof(src_ind->produced_cargo); cargo_index++) {
+		if (src_ind->produced_cargo[cargo_index] != CT_INVALID) num_cargos++;
 	}
+	if (num_cargos == 0) return false; // industry produces nothing
+	int cargo_num = RandomRange(num_cargos) + 1;
+	for (cargo_index = 0; cargo_index < lengthof(src_ind->produced_cargo); cargo_index++) {
+		if (src_ind->produced_cargo[cargo_index] != CT_INVALID) cargo_num--;
+		if (cargo_num == 0) break;
+	}
+	assert(cargo_num == 0); // indicates loop didn't break as intended
+	cid = src_ind->produced_cargo[cargo_index];
+	trans = src_ind->last_month_pct_transported[cargo_index];
+	total = src_ind->last_month_production[cargo_index];
 
 	/* Quit if no production in this industry
-	 * or if the pct transported is already large enough */
-	if (total == 0 || trans > SUBSIDY_MAX_PCT_TRANSPORTED || cid == CT_INVALID) return false;
+	 * or if the pct transported is already large enough
+	 * or if the cargo is automatically distributed */
+	if (total == 0 || trans > SUBSIDY_MAX_PCT_TRANSPORTED ||
+			cid == CT_INVALID ||
+			_settings_game.linkgraph.GetDistributionType(cid) != DT_MANUAL) {
+		return false;
+	}
 
 	SourceID src = src_ind->index;
 
@@ -399,7 +417,7 @@ bool FindSubsidyIndustryCargoRoute()
 /**
  * Tries to find a suitable destination for the given source and cargo.
  * @param cid      Subsidized cargo.
- * @param src_type Type of #src.
+ * @param src_type Type of \a src.
  * @param src      Index of source.
  * @return True iff the subsidy was created.
  */
@@ -424,14 +442,11 @@ bool FindSubsidyCargoDestination(CargoID cid, SourceType src_type, SourceID src)
 		case ST_INDUSTRY: {
 			/* Select a random industry. */
 			const Industry *dst_ind = Industry::GetRandom();
+			if (dst_ind == NULL) return false;
 
 			/* The industry must accept the cargo */
-			if (dst_ind == NULL ||
-					(cid != dst_ind->accepts_cargo[0] &&
-					 cid != dst_ind->accepts_cargo[1] &&
-					 cid != dst_ind->accepts_cargo[2])) {
-				return false;
-			}
+			bool valid = std::find(dst_ind->accepts_cargo, endof(dst_ind->accepts_cargo), cid) != endof(dst_ind->accepts_cargo);
+			if (!valid) return false;
 
 			dst = dst_ind->index;
 			break;
@@ -464,13 +479,13 @@ void SubsidyMonthlyLoop()
 		if (--s->remaining == 0) {
 			if (!s->IsAwarded()) {
 				Pair reftype = SetupSubsidyDecodeParam(s, true);
-				AddNewsItem(STR_NEWS_OFFER_OF_SUBSIDY_EXPIRED, NS_SUBSIDIES, (NewsReferenceType)reftype.a, s->src, (NewsReferenceType)reftype.b, s->dst);
+				AddNewsItem(STR_NEWS_OFFER_OF_SUBSIDY_EXPIRED, NT_SUBSIDIES, NF_NORMAL, (NewsReferenceType)reftype.a, s->src, (NewsReferenceType)reftype.b, s->dst);
 				AI::BroadcastNewEvent(new ScriptEventSubsidyOfferExpired(s->index));
 				Game::NewEvent(new ScriptEventSubsidyOfferExpired(s->index));
 			} else {
 				if (s->awarded == _local_company) {
 					Pair reftype = SetupSubsidyDecodeParam(s, true);
-					AddNewsItem(STR_NEWS_SUBSIDY_WITHDRAWN_SERVICE, NS_SUBSIDIES, (NewsReferenceType)reftype.a, s->src, (NewsReferenceType)reftype.b, s->dst);
+					AddNewsItem(STR_NEWS_SUBSIDY_WITHDRAWN_SERVICE, NT_SUBSIDIES, NF_NORMAL, (NewsReferenceType)reftype.a, s->src, (NewsReferenceType)reftype.b, s->dst);
 				}
 				AI::BroadcastNewEvent(new ScriptEventSubsidyExpired(s->index));
 				Game::NewEvent(new ScriptEventSubsidyExpired(s->index));
@@ -480,7 +495,16 @@ void SubsidyMonthlyLoop()
 		}
 	}
 
-	if (modified) RebuildSubsidisedSourceAndDestinationCache();
+	if (modified) {
+		RebuildSubsidisedSourceAndDestinationCache();
+	} else if (_settings_game.linkgraph.distribution_pax != DT_MANUAL &&
+			   _settings_game.linkgraph.distribution_mail != DT_MANUAL &&
+			   _settings_game.linkgraph.distribution_armoured != DT_MANUAL &&
+			   _settings_game.linkgraph.distribution_default != DT_MANUAL) {
+		/* Return early if there are no manually distributed cargoes and if we
+		 * don't need to invalidate the subsidies window. */
+		return;
+	}
 
 	bool passenger_subsidy = false;
 	bool town_subsidy = false;
@@ -488,7 +512,7 @@ void SubsidyMonthlyLoop()
 
 	int random_chance = RandomRange(16);
 
-	if (random_chance < 2) {
+	if (random_chance < 2 && _settings_game.linkgraph.distribution_pax == DT_MANUAL) {
 		/* There is a 1/8 chance each month of generating a passenger subsidy. */
 		int n = 1000;
 
@@ -520,7 +544,7 @@ void SubsidyMonthlyLoop()
  * Tests whether given delivery is subsidised and possibly awards the subsidy to delivering company
  * @param cargo_type type of cargo
  * @param company company delivering the cargo
- * @param src_type type of #src
+ * @param src_type type of \a src
  * @param src index of source
  * @param st station where the cargo is delivered to
  * @return is the delivery subsidised?
