@@ -1991,7 +1991,7 @@ static const char * const _name_romanian_real[] = {
 	"Motru",
 	"N\xC4\x83s\xC4\x83ud",
 	"N\xC4\x83vodari",
-	"Odobe\xC8x99ti",
+	"Odobe\xC8\x99ti",
 	"Olteni\xC8\x9B""a",
 	"One\xC8\x99ti",
 	"Oradea",
