@@ -1,4 +1,4 @@
-/* $Id: tree_cmd.cpp 23740 2012-01-03 21:32:51Z rubidium $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -29,6 +29,8 @@
 #include "table/tree_land.h"
 #include "table/clear_land.h"
 
+#include "safeguards.h"
+
 /**
  * List of tree placer algorithm.
  *
@@ -323,10 +325,10 @@ void GenerateTrees()
 
 /**
  * Plant a tree.
- * @param tile start tile of area-drag of tree plantation
+ * @param tile end tile of area-drag
  * @param flags type of operation
  * @param p1 tree type, TREE_INVALID means random.
- * @param p2 end tile of area-drag
+ * @param p2 start tile of area-drag of tree plantation
  * @param text unused
  * @return the cost of this operation or an error
  */
@@ -340,6 +342,9 @@ CommandCost CmdPlantTree(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 	/* Check the tree type within the current climate */
 	if (tree_to_plant != TREE_INVALID && !IsInsideBS(tree_to_plant, _tree_base_by_landscape[_settings_game.game_creation.landscape], _tree_count_by_landscape[_settings_game.game_creation.landscape])) return CMD_ERROR;
 
+	Company *c = (_game_mode != GM_EDITOR) ? Company::GetIfValid(_current_company) : NULL;
+	int limit = (c == NULL ? INT32_MAX : GB(c->tree_limit, 16, 16));
+
 	TileArea ta(tile, p2);
 	TILE_AREA_LOOP(tile, ta) {
 		switch (GetTileType(tile)) {
@@ -350,9 +355,16 @@ CommandCost CmdPlantTree(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 					continue;
 				}
 
+				/* Test tree limit. */
+				if (--limit < 1) {
+					msg = STR_ERROR_TREE_PLANT_LIMIT_REACHED;
+					break;
+				}
+
 				if (flags & DC_EXEC) {
 					AddTreeCount(tile, 1);
 					MarkTileDirtyByTile(tile);
+					if (c != NULL) c->tree_limit -= 1 << 16;
 				}
 				/* 2x as expensive to add more trees to an existing tile */
 				cost.AddCost(_price[PR_BUILD_TREES] * 2);
@@ -363,7 +375,8 @@ CommandCost CmdPlantTree(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 					msg = STR_ERROR_CAN_T_BUILD_ON_WATER;
 					continue;
 				}
-				/* FALL THROUGH */
+				FALLTHROUGH;
+
 			case MP_CLEAR: {
 				if (IsBridgeAbove(tile)) {
 					msg = STR_ERROR_SITE_UNSUITABLE;
@@ -383,6 +396,12 @@ CommandCost CmdPlantTree(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 					continue;
 				}
 
+				/* Test tree limit. */
+				if (--limit < 1) {
+					msg = STR_ERROR_TREE_PLANT_LIMIT_REACHED;
+					break;
+				}
+
 				if (IsTileType(tile, MP_CLEAR)) {
 					/* Remove fields or rocks. Note that the ground will get barrened */
 					switch (GetRawClearGround(tile)) {
@@ -412,6 +431,7 @@ CommandCost CmdPlantTree(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 					/* Plant full grown trees in scenario editor */
 					PlantTreesOnTile(tile, treetype, 0, _game_mode == GM_EDITOR ? 3 : 0);
 					MarkTileDirtyByTile(tile);
+					if (c != NULL) c->tree_limit -= 1 << 16;
 
 					/* When planting rainforest-trees, set tropiczone to rainforest in editor. */
 					if (_game_mode == GM_EDITOR && IsInsideMM(treetype, TREE_RAINFOREST, TREE_CACTUS)) {
@@ -426,6 +446,9 @@ CommandCost CmdPlantTree(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 				msg = STR_ERROR_SITE_UNSUITABLE;
 				break;
 		}
+
+		/* Tree limit used up? No need to check more. */
+		if (limit < 0) break;
 	}
 
 	if (cost.GetCost() == 0) {
@@ -572,7 +595,7 @@ static void TileLoopTreesDesert(TileIndex tile)
 			};
 			uint32 r = Random();
 
-			if (Chance16I(1, 200, r)) SndPlayTileFx(forest_sounds[GB(r, 16, 2)], tile);
+			if (Chance16I(1, 200, r) && _settings_client.sound.ambient) SndPlayTileFx(forest_sounds[GB(r, 16, 2)], tile);
 			break;
 		}
 
@@ -601,7 +624,7 @@ static void TileLoopTreesAlps(TileIndex tile)
 		} else {
 			if (GetTreeDensity(tile) == 3) {
 				uint32 r = Random();
-				if (Chance16I(1, 200, r)) {
+				if (Chance16I(1, 200, r) && _settings_client.sound.ambient) {
 					SndPlayTileFx((r & 0x80000000) ? SND_39_HEAVY_WIND : SND_34_WIND, tile);
 				}
 			}
@@ -658,7 +681,7 @@ static void TileLoop_Trees(TileIndex tile)
 							SetTreeGrowth(tile, 0);
 							break;
 						}
-						/* FALL THROUGH */
+						FALLTHROUGH;
 
 					case 2: { // add a neighbouring tree
 						/* Don't plant extra trees if that's not allowed. */
