@@ -1,4 +1,4 @@
-/* $Id: wnd_quartz.mm 27675 2016-10-31 19:29:01Z michi_cc $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -33,6 +33,7 @@
 #include "cocoa_v.h"
 #include "../../core/math_func.hpp"
 #include "../../gfx_func.h"
+#include "../../framerate_type.h"
 
 /* On some old versions of MAC OS this may not be defined.
  * Those versions generally only produce code for PPC. So it should be safe to
@@ -105,35 +106,6 @@ public:
 };
 
 
-static CGColorSpaceRef QZ_GetCorrectColorSpace()
-{
-	static CGColorSpaceRef colorSpace = NULL;
-
-	if (colorSpace == NULL) {
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
-		if (MacOSVersionIsAtLeast(10, 5, 0)) {
-			colorSpace = CGDisplayCopyColorSpace(CGMainDisplayID());
-		} else
-#endif
-		{
-#if (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5) && !defined(HAVE_OSX_1011_SDK)
-			CMProfileRef sysProfile;
-			if (CMGetSystemProfile(&sysProfile) == noErr) {
-				colorSpace = CGColorSpaceCreateWithPlatformColorSpace(sysProfile);
-				CMCloseProfile(sysProfile);
-			}
-#endif
-		}
-
-		if (colorSpace == NULL) colorSpace = CGColorSpaceCreateDeviceRGB();
-
-		if (colorSpace == NULL) error("Could not get system colour space. You might need to recalibrate your monitor.");
-	}
-
-	return colorSpace;
-}
-
-
 @implementation OTTD_QuartzView
 
 - (void)setDriver:(WindowQuartzSubdriver*)drv
@@ -333,7 +305,9 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height, int bpp)
 		[ this->window setAcceptsMouseMovedEvents:YES ];
 		[ this->window setViewsNeedDisplay:NO ];
 
-		[ this->window useOptimizedDrawing:YES ];
+#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_10
+		if ([ this->window respondsToSelector:@selector(useOptimizedDrawing:) ]) [ this->window useOptimizedDrawing:YES ];
+#endif
 
 		delegate = [ [ OTTD_CocoaWindowDelegate alloc ] init ];
 		[ delegate setDriver:this ];
@@ -429,6 +403,8 @@ WindowQuartzSubdriver::~WindowQuartzSubdriver()
 
 void WindowQuartzSubdriver::Draw(bool force_update)
 {
+	PerformanceMeasurer framerate(PFE_VIDEO);
+
 	/* Check if we need to do anything */
 	if (this->num_dirty_rects == 0 || [ this->window isMiniaturized ]) return;
 
@@ -517,7 +493,16 @@ CGPoint WindowQuartzSubdriver::PrivateLocalToCG(NSPoint *p)
 	p->y = this->window_height - p->y;
 	*p = [ this->cocoaview convertPoint:*p toView:nil ];
 
-	*p = [ this->window convertBaseToScreen:*p ];
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
+	if ([ this->window respondsToSelector:@selector(convertRectToScreen:) ]) {
+		*p = [ this->window convertRectToScreen:NSMakeRect(p->x, p->y, 0, 0) ].origin;
+	} else
+#endif
+	{
+#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_7
+		*p = [ this->window convertBaseToScreen:*p ];
+#endif
+	}
 	p->y = this->device_height - p->y;
 
 	CGPoint cgp;
@@ -533,13 +518,15 @@ NSPoint WindowQuartzSubdriver::GetMouseLocation(NSEvent *event)
 
 	if ( [ event window ] == nil) {
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
-		if ([ this->cocoaview respondsToSelector:@selector(convertRectFromScreen:) ]) {
+		if ([ [ this->cocoaview window ] respondsToSelector:@selector(convertRectFromScreen:) ]) {
 			pt = [ this->cocoaview convertPoint:[ [ this->cocoaview window ] convertRectFromScreen:NSMakeRect([ event locationInWindow ].x, [ event locationInWindow ].y, 0, 0) ].origin fromView:nil ];
 		}
 		else
 #endif
 		{
+#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_7
 			pt = [ this->cocoaview convertPoint:[ [ this->cocoaview window ] convertScreenToBase:[ event locationInWindow ] ] fromView:nil ];
+#endif
 		}
 	} else {
 		pt = [ event locationInWindow ];
@@ -580,20 +567,36 @@ bool WindowQuartzSubdriver::WindowResized()
 	this->window_width = (int)newframe.size.width;
 	this->window_height = (int)newframe.size.height;
 
+	/* Get screen colour space. */
+	CGColorSpaceRef color_space = NULL;
+
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6)
+	if ([ this->window respondsToSelector:@selector(colorSpace) ]) {
+		color_space = [ [ this->window colorSpace ] CGColorSpace ];
+		CGColorSpaceRetain(color_space);
+	}
+#endif
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
+	if (color_space == NULL && MacOSVersionIsAtLeast(10, 5, 0)) color_space = CGColorSpaceCreateWithName(kCGColorSpaceSRGB);
+#endif
+	if (color_space == NULL) color_space = CGColorSpaceCreateDeviceRGB();
+	if (color_space == NULL) error("Could not get system colour space. You might need to recalibrate your monitor.");
+
 	/* Create Core Graphics Context */
 	free(this->window_buffer);
 	this->window_buffer = (uint32*)malloc(this->window_width * this->window_height * 4);
 
 	CGContextRelease(this->cgcontext);
 	this->cgcontext = CGBitmapContextCreate(
-		this->window_buffer,        // data
+		this->window_buffer,       // data
 		this->window_width,        // width
 		this->window_height,       // height
 		8,                         // bits per component
 		this->window_width * 4,    // bytes per row
-		QZ_GetCorrectColorSpace(), // color space
+		color_space,               // color space
 		kCGImageAlphaNoneSkipFirst | kCGBitmapByteOrder32Host
 	);
+	CGColorSpaceRelease(color_space);
 
 	assert(this->cgcontext != NULL);
 	CGContextSetShouldAntialias(this->cgcontext, FALSE);
