@@ -1,4 +1,4 @@
-/* $Id: win32_v.cpp 27935 2017-12-09 19:21:45Z michi_cc $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -23,6 +23,7 @@
 #include "../progress.h"
 #include "../window_gui.h"
 #include "../window_func.h"
+#include "../framerate_type.h"
 #include "win32_v.h"
 #include <windows.h>
 #include <imm.h>
@@ -38,6 +39,9 @@
 #define PM_QS_INPUT 0x20000
 #endif
 
+typedef BOOL (WINAPI *PFNTRACKMOUSEEVENT)(LPTRACKMOUSEEVENT lpEventTrack);
+static PFNTRACKMOUSEEVENT _pTrackMouseEvent = NULL;
+
 static struct {
 	HWND main_wnd;
 	HBITMAP dib_sect;
@@ -57,9 +61,7 @@ bool _force_full_redraw;
 bool _window_maximize;
 uint _display_hz;
 static Dimension _bck_resolution;
-#if !defined(WINCE) || _WIN32_WCE >= 0x400
 DWORD _imm_props;
-#endif
 
 /** Whether the drawing is/may be done in a separate thread. */
 static bool _draw_threaded;
@@ -236,7 +238,7 @@ int RedrawScreenDebug()
 #define TID_POLLMOUSE 1
 #define MOUSE_POLL_DELAY 75
 
-static void CALLBACK TrackMouseTimerProc(HWND hwnd, UINT msg, UINT event, DWORD time)
+static void CALLBACK TrackMouseTimerProc(HWND hwnd, UINT msg, UINT_PTR event, DWORD time)
 {
 	RECT rc;
 	POINT pt;
@@ -269,9 +271,6 @@ bool VideoDriver_Win32::MakeWindow(bool full_screen)
 		_wnd.main_wnd = 0;
 	}
 
-#if defined(WINCE)
-	/* WinCE is always fullscreen */
-#else
 	if (full_screen) {
 		DEVMODE settings;
 
@@ -315,7 +314,6 @@ bool VideoDriver_Win32::MakeWindow(bool full_screen)
 		_wnd.width = _bck_resolution.width;
 		_wnd.height = _bck_resolution.height;
 	}
-#endif
 
 	{
 		RECT r;
@@ -334,22 +332,20 @@ bool VideoDriver_Win32::MakeWindow(bool full_screen)
 			SetRect(&r, 0, 0, _wnd.width, _wnd.height);
 		}
 
-#if !defined(WINCE)
 		AdjustWindowRect(&r, style, FALSE);
-#endif
 		w = r.right - r.left;
 		h = r.bottom - r.top;
 
 		if (_wnd.main_wnd != NULL) {
 			if (!_window_maximize) SetWindowPos(_wnd.main_wnd, 0, 0, 0, w, h, SWP_NOACTIVATE | SWP_NOOWNERZORDER | SWP_NOZORDER | SWP_NOMOVE);
 		} else {
-			TCHAR Windowtitle[50];
 			int x = (GetSystemMetrics(SM_CXSCREEN) - w) / 2;
 			int y = (GetSystemMetrics(SM_CYSCREEN) - h) / 2;
 
-			_sntprintf(Windowtitle, lengthof(Windowtitle), _T("OpenTTD %s"), MB_TO_WIDE(_openttd_revision));
+			char window_title[64];
+			seprintf(window_title, lastof(window_title), "OpenTTD %s", _openttd_revision);
 
-			_wnd.main_wnd = CreateWindow(_T("OTTD"), Windowtitle, style, x, y, w, h, 0, 0, GetModuleHandle(NULL), 0);
+			_wnd.main_wnd = CreateWindow(_T("OTTD"), MB_TO_WIDE(window_title), style, x, y, w, h, 0, 0, GetModuleHandle(NULL), 0);
 			if (_wnd.main_wnd == NULL) usererror("CreateWindow failed");
 			ShowWindow(_wnd.main_wnd, showstyle);
 		}
@@ -364,6 +360,8 @@ bool VideoDriver_Win32::MakeWindow(bool full_screen)
 /** Do palette animation and blit to the window. */
 static void PaintWindow(HDC dc)
 {
+	PerformanceMeasurer framerate(PFE_VIDEO);
+
 	HDC dc2 = CreateCompatibleDC(dc);
 	HBITMAP old_bmp = (HBITMAP)SelectObject(dc2, _wnd.dib_sect);
 	HPALETTE old_palette = SelectPalette(dc, _wnd.gdi_palette, FALSE);
@@ -494,7 +492,6 @@ static LRESULT HandleCharMsg(uint keycode, WChar charcode)
 	return 0;
 }
 
-#if !defined(WINCE) || _WIN32_WCE >= 0x400
 /** Should we draw the composition string ourself, i.e is this a normal IME? */
 static bool DrawIMECompositionString()
 {
@@ -631,15 +628,6 @@ static void CancelIMEComposition(HWND hwnd)
 	HandleTextInput(NULL, true);
 }
 
-#else
-
-static bool DrawIMECompositionString() { return false; }
-static void SetCompositionPos(HWND hwnd) {}
-static void SetCandidatePos(HWND hwnd) {}
-static void CancelIMEComposition(HWND hwnd) {}
-
-#endif /* !defined(WINCE) || _WIN32_WCE >= 0x400 */
-
 static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
 	static uint32 keycode = 0;
@@ -648,11 +636,9 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 
 	switch (msg) {
 		case WM_CREATE:
-			SetTimer(hwnd, TID_POLLMOUSE, MOUSE_POLL_DELAY, (TIMERPROC)TrackMouseTimerProc);
+			SetTimer(hwnd, TID_POLLMOUSE, MOUSE_POLL_DELAY, TrackMouseTimerProc);
 			SetCompositionPos(hwnd);
-#if !defined(WINCE) || _WIN32_WCE >= 0x400
 			_imm_props = ImmGetProperty(GetKeyboardLayout(0), IGP_PROPERTY);
-#endif
 			break;
 
 		case WM_ENTERSIZEMOVE:
@@ -747,7 +733,16 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			 * tracking the mouse for exiting the window */
 			if (!_cursor.in_window) {
 				_cursor.in_window = true;
-				SetTimer(hwnd, TID_POLLMOUSE, MOUSE_POLL_DELAY, (TIMERPROC)TrackMouseTimerProc);
+				if (_pTrackMouseEvent != NULL) {
+					TRACKMOUSEEVENT tme;
+					tme.cbSize = sizeof(tme);
+					tme.dwFlags = TME_LEAVE;
+					tme.hwndTrack = hwnd;
+
+					_pTrackMouseEvent(&tme);
+				} else {
+					SetTimer(hwnd, TID_POLLMOUSE, MOUSE_POLL_DELAY, TrackMouseTimerProc);
+				}
 			}
 
 			if (_cursor.fix_at) {
@@ -772,7 +767,6 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			return 0;
 		}
 
-#if !defined(WINCE) || _WIN32_WCE >= 0x400
 		case WM_INPUTLANGCHANGE:
 			_imm_props = ImmGetProperty(GetKeyboardLayout(0), IGP_PROPERTY);
 			break;
@@ -808,7 +802,6 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			}
 			HandleCharMsg(0, GB(wParam, 0, 8));
 			return 0;
-#endif
 #endif
 
 		case WM_DEADCHAR:
@@ -902,7 +895,6 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			}
 			return 0;
 
-#if !defined(WINCE)
 		case WM_SIZING: {
 			RECT *r = (RECT*)lParam;
 			RECT r2;
@@ -960,7 +952,6 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			}
 			return TRUE;
 		}
-#endif
 
 /* needed for wheel */
 #if !defined(WM_MOUSEWHEEL)
@@ -991,7 +982,6 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			_wnd.has_focus = false;
 			break;
 
-#if !defined(WINCE)
 		case WM_ACTIVATE: {
 			/* Don't do anything if we are closing openttd */
 			if (_exit_game) break;
@@ -1011,7 +1001,6 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			}
 			break;
 		}
-#endif
 	}
 
 	return DefWindowProc(hwnd, msg, wParam, lParam);
@@ -1038,6 +1027,9 @@ static void RegisterWndClass()
 
 		registered = true;
 		if (!RegisterClass(&wnd)) usererror("RegisterClass failed");
+
+		/* Dynamically load mouse tracking, as it doesn't exist on Windows 95. */
+		_pTrackMouseEvent = (PFNTRACKMOUSEEVENT)GetProcAddress(GetModuleHandle(_T("User32")), "TrackMouseEvent");
 	}
 }
 
@@ -1097,10 +1089,6 @@ static const Dimension default_resolutions[] = {
 static void FindResolutions()
 {
 	uint n = 0;
-#if defined(WINCE)
-	/* EnumDisplaySettingsW is only supported in CE 4.2+
-	 * XXX -- One might argue that we assume 4.2+ on every system. Then we can use this function safely */
-#else
 	uint i;
 	DEVMODEA dm;
 
@@ -1130,7 +1118,6 @@ static void FindResolutions()
 			}
 		}
 	}
-#endif
 
 	/* We have found no resolutions, show the default list */
 	if (n == 0) {
@@ -1176,9 +1163,7 @@ void VideoDriver_Win32::Stop()
 	DeleteObject(_wnd.dib_sect);
 	DestroyWindow(_wnd.main_wnd);
 
-#if !defined(WINCE)
 	if (_wnd.fullscreen) ChangeDisplaySettings(NULL, 0);
-#endif
 	MyShowCursor(true);
 }
 
@@ -1280,10 +1265,8 @@ void VideoDriver_Win32::MainLoop()
 
 			if (old_ctrl_pressed != _ctrl_pressed) HandleCtrlChanged();
 
-#if !defined(WINCE)
 			/* Flush GDI buffer to ensure we don't conflict with the drawing thread. */
 			GdiFlush();
-#endif
 
 			/* The game loop is the part that can run asynchronously.
 			 * The rest except sleeping can't. */
@@ -1296,10 +1279,8 @@ void VideoDriver_Win32::MainLoop()
 			UpdateWindows();
 			CheckPaletteAnim();
 		} else {
-#if !defined(WINCE)
 			/* Flush GDI buffer to ensure we don't conflict with the drawing thread. */
 			GdiFlush();
-#endif
 
 			/* Release the thread while sleeping */
 			if (_draw_threaded) _draw_mutex->EndCritical();
