@@ -1,4 +1,4 @@
-/* $Id: dropdown.cpp 27863 2017-05-03 20:09:51Z frosch $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,6 +14,7 @@
 #include "../string_func.h"
 #include "../strings_func.h"
 #include "../window_func.h"
+#include "../guitimer_func.h"
 #include "dropdown_type.h"
 
 #include "dropdown_widget.h"
@@ -97,6 +98,7 @@ struct DropdownWindow : Window {
 	bool drag_mode;
 	bool instant_close;           ///< Close the window when the mouse button is raised.
 	int scrolling;                ///< If non-zero, auto-scroll the item list (one time).
+	GUITimer scrolling_timer;     ///< Timer for auto-scroll of the item list.
 	Point position;               ///< Position of the topleft corner of the window.
 	Scrollbar *vscroll;
 
@@ -111,7 +113,6 @@ struct DropdownWindow : Window {
 	 * @param size          Size of the dropdown menu window.
 	 * @param wi_colour     Colour of the parent widget.
 	 * @param scroll        Dropdown menu has a scrollbar.
-	 * @param widget        Widgets of the dropdown menu window.
 	 */
 	DropdownWindow(Window *parent, const DropDownList *list, int selected, int button, bool instant_close, const Point &position, const Dimension &size, Colours wi_colour, bool scroll)
 			: Window(&_dropdown_desc)
@@ -156,6 +157,7 @@ struct DropdownWindow : Window {
 		this->click_delay      = 0;
 		this->drag_mode        = true;
 		this->instant_close    = instant_close;
+		this->scrolling_timer  = GUITimer(MILLISECONDS_PER_TICK);
 	}
 
 	~DropdownWindow()
@@ -182,7 +184,7 @@ struct DropdownWindow : Window {
 
 	/**
 	 * Find the dropdown item under the cursor.
-	 * @param value [out] Selected item, if function returns \c true.
+	 * @param[out] value Selected item, if function returns \c true.
 	 * @return Cursor points to a dropdown item.
 	 */
 	bool GetDropDownItem(int &value)
@@ -255,8 +257,11 @@ struct DropdownWindow : Window {
 		}
 	}
 
-	virtual void OnTick()
+	virtual void OnRealtimeTick(uint delta_ms)
 	{
+		if (!this->scrolling_timer.Elapsed(delta_ms)) return;
+		this->scrolling_timer.SetInterval(MILLISECONDS_PER_TICK);
+
 		if (this->scrolling != 0) {
 			int pos = this->vscroll->GetPosition();
 
@@ -347,8 +352,8 @@ void ShowDropDownListAt(Window *w, const DropDownList *list, int selected, int b
 	/* Longest item in the list, if auto_width is enabled */
 	uint max_item_width = 0;
 
-	/* Total length of list */
-	int height = 0;
+	/* Total height of list */
+	uint height = 0;
 
 	for (const DropDownListItem * const *it = list->Begin(); it != list->End(); ++it) {
 		const DropDownListItem *item = *it;
@@ -356,53 +361,58 @@ void ShowDropDownListAt(Window *w, const DropDownList *list, int selected, int b
 		if (auto_width) max_item_width = max(max_item_width, item->Width() + 5);
 	}
 
-	/* Check if the status bar is visible, as we don't want to draw over it */
-	int screen_bottom = GetMainViewBottom();
+	/* Scrollbar needed? */
 	bool scroll = false;
 
-	/* Check if the dropdown will fully fit below the widget */
-	if (top + height + 4 >= screen_bottom) {
-		/* If not, check if it will fit above the widget */
-		int screen_top = GetMainViewTop();
-		if (w->top + wi_rect.top > screen_top + height) {
-			top = w->top + wi_rect.top - height - 4;
-		} else {
-			/* If it doesn't fit above the widget, we need to enable a scrollbar... */
-			int avg_height = height / (int)list->Length();
-			scroll = true;
+	/* Is it better to place the dropdown above the widget? */
+	bool above = false;
 
-			/* ... and choose whether to put the list above or below the widget. */
-			bool put_above = false;
-			int available_height = screen_bottom - w->top - wi_rect.bottom;
-			if (w->top + wi_rect.top - screen_top > available_height) {
-				// Put it above.
-				available_height = w->top + wi_rect.top - screen_top;
-				put_above = true;
-			}
+	/* Available height below (or above, if the dropdown is placed above the widget). */
+	uint available_height = (uint)max(GetMainViewBottom() - top - 4, 0);
+
+	/* If the dropdown doesn't fully fit below the widget... */
+	if (height > available_height) {
+
+		uint available_height_above = (uint)max(w->top + wi_rect.top - GetMainViewTop() - 4, 0);
+
+		/* Put the dropdown above if there is more available space. */
+		if (available_height_above > available_height) {
+			above = true;
+			available_height = available_height_above;
+		}
+
+		/* If the dropdown doesn't fully fit, we need a dropdown. */
+		if (height > available_height) {
+			scroll = true;
+			uint avg_height = height / list->Length();
 
 			/* Check at least there is space for one item. */
 			assert(available_height >= avg_height);
 
-			/* And lastly, fit the list... */
-			int rows = available_height / avg_height;
+			/* Fit the list. */
+			uint rows = available_height / avg_height;
 			height = rows * avg_height;
 
-			/* Add space for the scroll bar if we automatically determined
-			 * the width of the list. */
+			/* Add space for the scrollbar. */
 			max_item_width += NWidgetScrollbar::GetVerticalDimension().width;
+		}
 
-			/* ... and set the top position if needed. */
-			if (put_above) {
-				top = w->top + wi_rect.top - height - 4;
-			}
+		/* Set the top position if needed. */
+		if (above) {
+			top = w->top + wi_rect.top - height - 4;
 		}
 	}
 
 	if (auto_width) width = max(width, max_item_width);
 
 	Point dw_pos = { w->left + (_current_text_dir == TD_RTL ? wi_rect.right + 1 - (int)width : wi_rect.left), top};
-	Dimension dw_size = {width, (uint)height};
-	new DropdownWindow(w, list, selected, button, instant_close, dw_pos, dw_size, wi_colour, scroll);
+	Dimension dw_size = {width, height};
+	DropdownWindow *dropdown = new DropdownWindow(w, list, selected, button, instant_close, dw_pos, dw_size, wi_colour, scroll);
+
+	/* The dropdown starts scrolling downwards when opening it towards
+	 * the top and holding down the mouse button. It can be fooled by
+	 * opening the dropdown scrolled to the very bottom.  */
+	if (above && scroll) dropdown->vscroll->UpdatePosition(INT_MAX);
 }
 
 /**
