@@ -1,4 +1,4 @@
-/* $Id: statusbar_widget.h 23600 2011-12-19 20:46:17Z truebrain $ */
+/* $Id$ */
 
 /*
  * This file is part of OpenTTD.
@@ -13,7 +13,7 @@
 #define WIDGETS_STATUSBAR_WIDGET_H
 
 /** Widgets of the #StatusBarWindow class. */
-enum StatusbarWidget {
+enum StatusbarWidgets {
 	WID_S_LEFT,   ///< Left part of the statusbar; date is shown there.
 	WID_S_MIDDLE, ///< Middle part; current news or company name or *** SAVING *** or *** PAUSED ***.
 	WID_S_RIGHT,  ///< Right part; bank balance.
